Package: mirTarRnaSeq
Type: Package
Title: mirTarRnaSeq
Version: 1.20.0
Authors@R: 
    c(person(given = "Mercedeh",
           family = "Movassagh",
           role = c("aut", "cre"),
           email = "mercedeh@ds.dfci.harvard.edu",
           comment = c(ORCID = "0000-0001-7690-0230")),
           person("Sarah","Morton", role="aut"),
           person("Rafael","Irizarry", role="aut"),
           person("Jeffrey", "Bailey", role= "aut"),
           person("Joseph N", "Paulson", email="paulson.joseph@gene.com", role="aut"))
Description: mirTarRnaSeq R package can be used for interactive mRNA miRNA sequencing statistical analysis. This package utilizes expression or differential expression mRNA and miRNA sequencing results and performs interactive correlation and various GLMs (Regular GLM, Multivariate GLM, and Interaction GLMs ) analysis between mRNA and miRNA expriments. These experiments can be time point experiments, and or condition expriments.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Depends: R (>= 4.1.0), ggplot2
DeploySubPath: mirTarRnaSeq
biocViews: miRNA, Regression, Software, Sequencing, SmallRNA,
        TimeCourse, DifferentialExpression
RoxygenNote: 7.2.3
Suggests: BiocStyle, knitr, rmarkdown, R.cache, SPONGE
VignetteBuilder: knitr
Imports: purrr, MASS, pscl, assertthat, caTools, dplyr, pheatmap,
        reshape2, corrplot, grDevices, graphics, stats, utils,
        data.table, R.utils, viridis
git_url: https://git.bioconductor.org/packages/mirTarRnaSeq
git_branch: RELEASE_3_23
git_last_commit: ff41128
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:50:51 UTC; biocbuild
Author: Mercedeh Movassagh [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7690-0230>),
  Sarah Morton [aut],
  Rafael Irizarry [aut],
  Jeffrey Bailey [aut],
  Joseph N Paulson [aut]
Maintainer: Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>
Built: R 4.6.0; ; 2026-04-29 04:50:51 UTC; unix

# Generated by roxygen2: do not edit by hand

export(combiner)
export(corMirnaRna)
export(corMirnaRnaMiranda)
export(drawCorPlot)
export(drawInterPlots)
export(fdrSig)
export(finInterResult)
export(geneVari)
export(getInputSpecies)
export(glm_gaussian)
export(glm_multi)
export(glm_nb)
export(glm_poisson)
export(glm_zeroinfl)
export(glm_zeroinfl_negbin)
export(glm_zeroinfl_poisson)
export(miRanComp)
export(miRandaIntersect)
export(mirRnaDensityCor)
export(mirRnaDensityInter)
export(mirRnaHeatmap)
export(mirRnaHeatmapDiff)
export(mirandaIntersectInter)
export(miranda_sponge_predict)
export(modelAIC)
export(modelCoefficients)
export(modelData)
export(modelModelName)
export(modelModelPvalue)
export(modelTermPvalues)
export(modelsFilter)
export(one2OneRnaMiRNA)
export(one2manySponge)
export(plotFit)
export(plotResiduals)
export(plotTerms)
export(runAllMirnaModels)
export(runModel)
export(runModels)
export(sampCorRnaMirna)
export(threshSig)
export(threshSigInter)
export(twoTimePoint)
export(twoTimePointSamp)
export(tzTrans)
import(R.utils)
importFrom(MASS,glm.nb)
importFrom(assertthat,assert_that)
importFrom(caTools,combs)
importFrom(corrplot,corrplot)
importFrom(data.table,fread)
importFrom(dplyr,`%>%`)
importFrom(dplyr,filter)
importFrom(dplyr,mutate)
importFrom(dplyr,select)
importFrom(grDevices,colorRampPalette)
importFrom(graphics,abline)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(pheatmap,pheatmap)
importFrom(pscl,zeroinfl)
importFrom(pscl,zeroinfl.control)
importFrom(purrr,is_formula)
importFrom(reshape2,dcast)
importFrom(stats,AIC)
importFrom(stats,cor)
importFrom(stats,density)
importFrom(stats,deviance)
importFrom(stats,df.residual)
importFrom(stats,formula)
importFrom(stats,gaussian)
importFrom(stats,glm)
importFrom(stats,glm.control)
importFrom(stats,lm)
importFrom(stats,model.frame)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,pf)
importFrom(stats,poisson)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,terms)
importFrom(stats,var)
importFrom(utils,download.file)
importFrom(utils,read.table)
importFrom(viridis,inferno)

<img src="man/figures/logo.png" align="right" alt="logo.png" width="180" />

# mirTarRnaSeq

<!-- badges: start -->
[![BioC status](http://www.bioconductor.org/shields/build/release/bioc/mirTarRnaSeq.svg)](https://bioconductor.org/checkResults/release/bioc-LATEST/mirTarRnaSeq)
<!-- badges: end -->

## Installation
```
devtools::install_github('Mercedeh66/mirTarRnaSeq')
```
## Issues and bug reports
Please use https://github.com/Mercedeh66/mirTarRnaSeq/issues to submit issues, bug reports, and comments.

## ----style, echo=FALSE, results='asis'----------------------------------------
BiocStyle::markdown()

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  dpi=300,
  warning = FALSE,
  collapse = TRUE,
  error = FALSE,
  comment = "#>",
  echo=TRUE
)
library(mirTarRnaSeq)
library(knitr)
library(rmarkdown)
library(mirTarRnaSeq)
library(dplyr)
library(ggplot2)
library(reshape2)
library(tidyr)
library(viridis)

doctype <- opts_knit$get("rmarkdown.pandoc.to")

## ----eval=TRUE, echo=TRUE-----------------------------------------------------
# Helper function to access test files in extdata
get_test_file <- function(file_name) {
  if (missing(file_name)) {
    return(system.file("extdata", "test", package="mirTarRnaSeq"))
  }
  return(system.file("extdata", "test", file_name, package="mirTarRnaSeq"))
}

# Read input files
DiffExp<-read.table(get_test_file("EBV_mRNA.txt.gz"), as.is=TRUE, header=TRUE, row.names=1)
miRNAExp<-read.table(get_test_file("EBV_miRNA.txt.gz"), as.is=TRUE, header=TRUE, row.names=1)

## ----eval=TRUE, echo=TRUE-----------------------------------------------------
miRanda <- getInputSpecies("Epstein_Barr", threshold = 140)
DiffExpmRNASub <- miRanComp(DiffExp, miRanda)

## ----eval=TRUE, echo=TRUE-----------------------------------------------------
miRNA_select<-c("ebv-mir-BART9-5p")

## ----eval=TRUE, echo=TRUE-----------------------------------------------------
Combine <- combiner(DiffExp, miRNAExp, miRNA_select)
geneVariant <- geneVari(Combine, miRNA_select)

## ----eval=TRUE, echo=TRUE-----------------------------------------------------
j <- runModel(`LMP-1` ~ `ebv-mir-BART9-5p`,
              Combine, model = glm_poisson(),
              scale = 100)
# Print P value of poisson model of association between between LMP_1 and
# ebv-mir-BART9-5p
print(modelTermPvalues(j))

## ----gauss, eval=TRUE, echo=TRUE,out.width="90%",out.height="90%", fig.align="center"----
blaGaus <- runModels(Combine,
                     geneVariant, miRNA_select,
                     family = glm_gaussian(),
                     scale = 100)
# Plot the regression relationship for the mRNA and miRNAs (BHLF1 is the EBV
# mRNA). Note, these are standard quality check plot outputs from plot
# regression.
par(oma=c(2,2,2,2))
par(mfrow=c(2,3),mar=c(4,3,3,2))
plot(blaGaus[["all_models"]][["BHLF1"]])
plot(modelData(blaGaus[["all_models"]][["BHLF1"]]))
# To comprehend the performance of all models we look at Akaike information
# criterion (AIC) across all miRNA-mRNA model performances and then look at
# the density plots. Note, the models with lower comparitive AICs have better
# performace.
G <- do.call(rbind.data.frame, blaGaus[["AICvalues"]])
names(G) <- c("AIC_G")
# Low values seems like a reasonable model
plot(density(G$AIC_G))
# Print out the AIC of all miRNA-mRNA models ( All observed AIC values for the
# miRNA-mRNA models)
GM <- melt(G)

## ----poisson, eval=TRUE, echo=TRUE, out.width="90%",out.height="90%", fig.align="center"----
blaPois <- runModels(Combine,
                     geneVariant, miRNA_select,
                     family = glm_poisson(),
                     scale = 100)
par(oma=c(2,2,2,2))
par(mfrow=c(2,3),mar=c(4,3,3,2))
plot(blaPois[["all_models"]][["LMP-2A"]])
plot(modelData(blaPois[["all_models"]][["LMP-2A"]]))
P <- do.call(rbind.data.frame, blaPois[["AICvalues"]])
names(P) <- c("AIC_Po")
PM <- melt(P)

## ----negbin, eval=TRUE, echo=TRUE, out.width="70%",out.height="90%", fig.align="center"----
blaNB <- runModels(Combine,
                   geneVariant, miRNA_select,
                   family = glm_nb(), scale = 100)
par(mar=c(4,3,3,2))
plot(modelData(blaNB[["all_models"]][["BALF1"]]))
B <- do.call(rbind.data.frame, blaNB[["AICvalues"]])
names(B) <- c("AIC_NB")
BM <- melt(B)

## ----zeroinfnegbin, eval=TRUE, echo=TRUE, out.width="70%",out.height="90%", fig.align="center"----
blazeroinflNB <- runModels(Combine, geneVariant,
                            miRNA_select,
                            family = glm_zeroinfl(dist = "negbin"),
                            scale = 100)
# To test AIC model performance
ZNB <- do.call(rbind.data.frame, blazeroinflNB[["AICvalues"]])
names(ZNB) <- c("AIC_ZNB")
par(mar=c(4,3,3,2))
plot(density(ZNB$AIC_ZNB))
ZNBM<-melt(ZNB)

## ----zeroinfpoisson, message=FALSE, echo=TRUE, cache=FALSE, results=TRUE, out.width="70%",out.height="90%", fig.align="center"----
blazeroinfl <- runModels(Combine, geneVariant,
                         miRNA_select,
                         family = glm_zeroinfl(),
                         scale = 100)
# To test AIC model performance
Zp <- do.call(rbind.data.frame, blazeroinfl[["AICvalues"]])
names(Zp) <- c("AIC_Zp")
par(mar=c(4,3,3,2))
plot(density(Zp$AIC_Zp))
ZpM <- melt(Zp)

## ----plots, eval=TRUE, echo=TRUE, fig.width = 5, fig.height=5, out.width="80%", dpi=300, fig.align="center"----
bindM <- rbind(PM, BM, GM, ZpM, ZNBM)
p2 <- ggplot(data = bindM, aes(x = value, group = variable,
                               fill = variable)) +
  geom_density(adjust = 1.5, alpha = .3) +
  xlim(-400, 2000)+
  ggtitle("Plot of of AIC for ebv-mir-BART9-5p regressed all mRNAs ")+
  ylab("Density")+ xlab ("AIC Value")
p2

## ----message=FALSE,echo=TRUE, cache=FALSE, results=TRUE, warning=FALSE, comment=TRUE, warning=FALSE----
miRNA_select<-c("ebv-mir-BART9-5p","ebv-mir-BART6-3p")
Combine <- combiner(DiffExp, miRNAExp, miRNA_select)
geneVariant <- geneVari(Combine, miRNA_select)
MultiModel <- runModels(Combine, geneVariant,
                        miRNA_select, family = glm_multi(),
                        mode="multi", scale = 10)
# Print the name of the models used for the analysis (note the printed outputs
# are the number of models ran by various models based on the AIC scores using
# the glm_multi())
print(table(unlist(lapply(MultiModel$all_models, modelModelName))))

## ----message=FALSE,echo=TRUE, cache=FALSE, results=TRUE, warning=FALSE, comment=TRUE, warning=FALSE----
miRNA_select<-c("ebv-mir-BART9-5p","ebv-mir-BART6-3p")
Combine <- combiner(DiffExp, miRNAExp, miRNA_select)
geneVariant <- geneVari(Combine, miRNA_select)
InterModel <- runModels(Combine,
                        geneVariant, miRNA_select,
                        family = glm_multi(
                          models=list(glm_gaussian,
                          glm_poisson())),mode="inter", scale = 10)

# Print the name of the models used for the analysis (note, you can see although
# we have defined for the models to be run using either gaussian or poisson
# options, mirTarRnaSeq chooses the poisson model for all as it performs with a
# better AIC (lower AIC value) for all miRNA-mRNA interactions)
print(table(unlist(lapply(InterModel$all_models, modelModelName))))

## ----message=FALSE,echo=TRUE, cache=FALSE, results=TRUE, warning=FALSE, comment=TRUE----
vMiRNA<-rownames(miRNAExp)
# Note, the user can run all miRNAs but for speed reasons we have chosen the
# first 5 here for mirnas input for the analysis.
All_miRNAs_run<-runAllMirnaModels(mirnas =vMiRNA[1:5] ,
                                  DiffExpmRNA = DiffExpmRNASub,
                                  DiffExpmiRNA = miRNAExp,
                                  miranda_data = miRanda,prob=0.75,
                                  cutoff=0.05,fdr_cutoff = 0.1, method = "fdr",
                                  family = glm_multi(), scale = 2, mode="multi")

#select significant genes
hasgenes <- lapply(All_miRNAs_run, function(x) nrow(x$SigFDRGenes)) > 0
All_miRNAs_run <- All_miRNAs_run[hasgenes]
print(table(unlist(lapply
                   (All_miRNAs_run[[1]][["FDRModel"]][["all_models"]],
                     modelModelName))))
# Print specific models for specific miRNAs (in this example the significant
# multivariate model for ebv-mir-BART1-5p and ebv-mir-BART11-3p )
print(
  table(
    unlist(lapply(
      (All_miRNAs_run[["ebv-mir-BART1-5p and ebv-mir-BART11-3p"]]
                     [["FDRModel"]]
                     [["all_models"]]),
        modelModelName))))

## ----eval=TRUE, echo=TRUE, results='hide', error=TRUE-------------------------
try({
# Make miRanda file compatible with SPONGE package mir_predicted_targets()
sponge_mir_input<-miranda_sponge_predict(miRNAExp,DiffExp,miRanda)

#Perform sparse partial correlation for miRNA-mRNA relationships
one2many_output<-one2manySponge(miRNAExp,DiffExp,sponge_mir_input)
})

## ----eval=TRUE, echo=TRUE-----------------------------------------------------
files <- local({
  filenames <- list.files(path=get_test_file(), pattern="^.*\\.txt\\.gz$",
                          full.names=TRUE)
  files <- lapply(filenames, read.table, as.is=TRUE, header=TRUE, sep="\t")
  names(files) <- gsub("^.*/(.*)\\.txt\\.gz$", "\\1", filenames)
  return(files)
})

## ----eval=TRUE, echo=TRUE-----------------------------------------------------
mrna_files <- files[grep("^mRNA", names(files))]

## ----eval=TRUE, echo=TRUE-----------------------------------------------------
mrna_files <- files[grep("^mRNA", names(files))]
mrna <- one2OneRnaMiRNA(mrna_files, pthreshold = 0.05)$foldchanges

## ----eval=TRUE, echo=TRUE-----------------------------------------------------
mirna_files <- files[grep("^miRNA", names(files))]
mirna <- one2OneRnaMiRNA(mirna_files)$foldchanges

## ----eval=TRUE, echo=TRUE-----------------------------------------------------
corr_0 <- corMirnaRna(mrna, mirna,method="pearson")

## ----eval=TRUE, echo=TRUE, results=FALSE--------------------------------------
outs <- sampCorRnaMirna(mrna, mirna,method="pearson",
                        Shrounds = 100, Srounds = 1000)

## ----eval=TRUE, echo=TRUE, fig.width = 3, fig.height=3, out.width="80%", dpi=300, fig.align="center"----
#Draw density plot
mirRnaDensityCor(corr_0, outs)

## ----eval=TRUE, echo=TRUE-----------------------------------------------------
#Identify significant correlation
sig_corrs <- threshSig(corr_0, outs,pvalue = 0.05)

## ----eval=TRUE, echo=TRUE-----------------------------------------------------
#Import concordant miRanda file
miRanda <- getInputSpecies("Mouse", threshold = 150)

## ----eval=TRUE, echo=TRUE, fig.width = 8, fig.height=5, out.width="90%", dpi=300, fig.align="center"----
#Extract your target correlations based on miRanda and correlation threshold.
newcorr <- corMirnaRnaMiranda(mrna, mirna, -0.7, miRanda)
mirRnaHeatmap(newcorr,upper_bound = -0.6)

## ----eval=TRUE, echo=TRUE, fig.width = 5, fig.height=4, out.width="80%", dpi=300, fig.align="center"----
#Make final results file for significant
#correlations intersecting with miRanda file
results <- miRandaIntersect(sig_corrs, outs, mrna, mirna, miRanda)
#Draw correlation heatmap
p<- mirRnaHeatmap(results$corr,upper_bound =-0.99)
p

## ----eval=TRUE, echo=TRUE-----------------------------------------------------
files <- local({
  filenames <- list.files(path=get_test_file(), pattern="^.*\\.txt\\.gz$",
                          full.names=TRUE)
  files <- lapply(filenames, read.table, as.is=TRUE, header=TRUE, sep="\t")
  names(files) <- gsub("^.*/(.*)\\.txt\\.gz$", "\\1", filenames)
  return(files)
})

## ----eval=TRUE, echo=TRUE-----------------------------------------------------
mirna_files <- files[grep("^miRNA0_5", names(files))]
mrna_files <- files[grep("^mRNA0_5", names(files))]

## ----eval=TRUE, echo=TRUE-----------------------------------------------------
# Parse Fold Change Files for P value and Fold Change.
mrna <- one2OneRnaMiRNA(mrna_files, pthreshold = 0.05)$foldchanges
mirna <- one2OneRnaMiRNA(mirna_files)$foldchanges

## ----eval=TRUE, echo=TRUE-----------------------------------------------------
# Estimate the miRNA mRNA FC differences for your dataset
inter0 <- twoTimePoint(mrna, mirna)

## ----eval=TRUE, echo=TRUE, message=FALSE, results=FALSE-----------------------
#Make a background distribution for your miRNA mRNA FC differences
outs <- twoTimePointSamp(mrna, mirna,Shrounds = 10 )

## ----eval=TRUE, echo=TRUE-----------------------------------------------------
#Import concordant miRanda file
miRanda <- getInputSpecies("Mouse", threshold = 140)

## ----eval=TRUE, echo=TRUE-----------------------------------------------------
#Identify miRNA mRNA relationships bellow a P value threshold, default is 0.05
sig_InterR <- threshSigInter(inter0, outs)

## ----eval=TRUE, echo=TRUE-----------------------------------------------------
#Intersect the mirRanda file with your output results
results <- mirandaIntersectInter(sig_InterR, outs, mrna, mirna, miRanda)

## ----eval=TRUE, echo=TRUE, fig.width = 4, fig.height=4, out.width="90%", dpi=300, fig.align="center"----
#Create a results file for heatmap
final_results <- finInterResult(results)
#Draw plots of miRNA mRNA fold changes for your results file
par(mar=c(4,4,2,1))
drawInterPlots(mrna,mirna,final_results)

## ----eval=TRUE, echo=TRUE, fig.width = 3, fig.height=2, out.width="80%", dpi=300, fig.align="center"----
CorRes<-results$corrs
#Draw heatmap for miRNA mRNA significant differences
#Note: you do not have to use the upper_bound function unless you want
#investigate a particular range for miRNA mRNA differences/relationships
mirRnaHeatmapDiff(CorRes,upper_bound = 9.9)

#' @importFrom dplyr `%>%` filter select mutate
#' @importFrom reshape2 dcast
NULL

#' Transform miRanda data for relevant mRNA and miRNA to matrix form compatible with sponge
#'
#' Transforms miRanda data into adjacency matrix, with 1 indicating
#' presence of a relationship between a mRNA and miRNA, and 0
#' otherwise. miRanda input is filtered by miRNA and mRNA present in
#' `mirna_exp` and `diff_expr` row names, respectively.
#'
#' @param mirna_exp miRna expression data.frame with miRNA for rows and samples for columns
#' @param diff_exp mRNA expression data.frame with mRNA for rows and samples for columns
#' @param miranda_data miRanda data.frame with the first two columns having miRNA and mRNA names
#' @return matrix adjacency matrix with column names miRNA and row names mRNA
#' @keywords miRanda, sparse partial correlation, ceRNA, SPONGE
#' @export
miranda_sponge_predict <- function(mirna_exp, diff_exp, miranda_data) {
  # ensure all inputs are data.frames
  stopifnot("mirna_exp not a data.frame"=is.data.frame(mirna_exp))
  stopifnot("diff_exp not a data.frame"=is.data.frame(diff_exp))
  stopifnot("miranda_data not a data.frame"=is.data.frame(miranda_data))

  # Ensure column names for miRanda are what we expect
  miranda_data <- miranda_data[, c(1, 2)] %>% mutate(V3=1L)
  colnames(miranda_data) <- paste0("V", seq.int(1, ncol(miranda_data)))

  # Add rows for both mrna and mirna to miranda, to ensure all combinations
  # exists in final "dcasted" output
  miranda_data <- miranda_data %>%
                  rbind(data.frame(V1=rownames(mirna_exp),
                                   V2=rownames(diff_exp)[1],
                                   V3=0L)) %>%
                  rbind(data.frame(V1=rownames(mirna_exp)[1],
                                   V2=rownames(diff_exp),
                                   V3=0L))

  # dcast into adjacency matrix, only for combinations of miRNA and mRNA
  # that exist in expressio data
  sponge_mtx <- miranda_data %>%
    filter(V2 %in% rownames(diff_exp) & V1 %in% rownames(mirna_exp))

  if (nrow(sponge_mtx) == 0 || ncol(sponge_mtx) == 0) {
    stop("No interactions present for given input matrices")
  }

  # dcast into adjacency matrix
  sponge_mtx <- dcast(sponge_mtx, V2 ~ V1, max, fill=0L, value.var="V3")

  # Use V1 for rownames
  rownames(sponge_mtx) <- sponge_mtx$V2
  sponge_mtx <- sponge_mtx %>% select(-V2)

  if (nrow(sponge_mtx) == 0 || ncol(sponge_mtx) == 0) {
    stop("No interactions present for given input matrices")
  }

  return(as.matrix(sponge_mtx))
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#' Model functions for GLM with negative binomial family.
#'
#' Runs models 'glm_gaussian', 'glm_nb', 'glm_poisson',
#' 'glm_zeroinfl(poisson)', 'glm_zeroinfl(negbin)' and returns
#' mode with lowest AIC.
#' @param models Model type, one or more of glm_gaussian, glm_nb, glm_poisson, glm_zeroinfl_poisson or glm_zeroinfl_negbin
#' @return structure containing functions \code{fit}, \code{coefficients},
#'         \code{aic}, \code{data}, \code{pterm}, \code{pmodel}, and a
#'         character string "glm_multi" in \code{model}.
#' @export
#' @examples
#' x <- glm_multi()
glm_multi <- function(models = c(
                          glm_gaussian, glm_nb, glm_poisson,
                          glm_zeroinfl_poisson, glm_zeroinfl_negbin
                      )) {

    # fit
    fit <- function(x, data, ...) {
        bestaic <- NULL
        bestmodel <- NULL
        for (m in models) {
            model <- runModel(x, data, model = m, ...)
            if (!is.null(model)) {
                currentaic <- modelAIC(model)
                if (is.null(bestaic) || currentaic < bestaic) {
                    bestaic <- currentaic
                    bestmodel <- model
                }
            }
        }
        return(bestmodel)
    }

    structure(list(
        fit = fit, coefficients = modelCoefficients, aic = modelAIC, data = modelData,
        pterm = modelTermPvalues, pmodel = modelModelPvalue, model = "glm_multi"
    ))
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#' makeFormulaRightSide makes right hand side of formula for model variables: vector of indep. variables
#'
#' This function make right hand side of formula for model variables: vector of indep. variables (i.e. miRNAs)
#' mode: 'multi' for simple, 'inter' for model with interactions returns a string in the form "~ a + b", or "~ a + b + a * b"
#' @param variables The vector created by miRNA_select
#' @param mode One of "multi", "inter" or NULL
#' @return data.frame containing Miranda data
#' @examples
#' \donttest{
#' x <- makeFormulaRightSide(variables, mode = "multi")
#' }
#'
makeFormulaRightSide <- function(variables, mode = "multi") {
    # be sure to properly quote variable names: `varname`.
    if (is.null(mode)) {
        mode <- "multi"
    }
    replace <- !grepl("^`.*`$", variables) # all that don't look like `somevar`
    variables[replace] <- paste("`", variables[replace], "`", sep = "")
    rightside <- paste(variables, collapse = " + ") # "~ a + b"
    if ("inter" == mode) {
        for (i in seq_len(length(variables) - 1)) {
            for (j in seq(i + 1, length(variables))) {
                rightside <- paste(rightside, paste(variables[i], variables[j], sep = " * "), sep = " + ") # "~ a + b + a * b"
            }
        }
    }
    return(rightside)
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#'  tzTransTranspose and z-score transformation
#'
#' Transposes and z-score transforms a matrix or data.frame.
#' @param x matrix of miRNA or mRNA or the data frame to be transformed
#' @return transposed and transformed version of x as a matrix.
#' @export
#' @keywords zscore, scale, transform, transpose
#' @examples
#' x <- tzTrans(miRNA)
tzTrans <- function(x) {
    tx <- t(x)
    tnorm1 <- apply(tx, 1, scale)
    col1 <- colnames(tx)
    rownames(tnorm1) <- col1
    tnorm <- as.data.frame(tnorm1)
    return(tnorm)
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#' @importFrom graphics mtext
#' @importFrom stats pf
NULL

#' plotTerms
#'
#' Plot terms description
#' @param model linear model
#' @return does not return value
#' @export
#' @keywords plot, residuals, model
#' @examples
#' plotTerms(lm(x ~ y, data = data.frame(x = runif(10), y = runif(10))))
plotTerms <- function(model) {
    m <- model$model
    plot(m, main = as.character.formula(model$terms))
    k <- length(model$coefficients) - 1 # Number of mirnas in model!
    n <- length(model$fitted.values)
    rs <- summary(model)$r.squared
    Pvalue <- pf((rs / k) / ((1 - rs) / (n - 1 - k)), k, n - 1 - k, lower.tail = FALSE) # p value for R-squared

    mtext(sprintf("P value of model fit: %.2g", Pvalue), line = .74)
    mtext(sprintf("R-squared of model fit: %.2f", summary(model)$r.squared), line = .01)
    return(invisible(NULL))
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#' @importFrom assertthat assert_that
#' @importFrom purrr is_formula
#' @importFrom stats terms
NULL

#' Run a model of a specific kind
#'
#' @param x model formula
#' @param data data.frame to run the model on
#' @param ... passed on to \code{fit()}
#' @param model model type
#' @return fitted model
#' @export
runModel <- function(x, data, ..., model = glm_gaussian()) {
    assert_that(is.character(x) || is_formula(x))
    assert_that(is.data.frame(data))
    # obtain model and extract member functions
    model <- canonicalModel_(model)
    fit_ <- model$fit
    ret <- fit_(x, data, ...)
    return(ret)
}

#' Obtain coefficients
#'
#' @param x fitted model
#' @return fitted model coefficients
#' @export
#' @examples
#' modelCoefficients(some_model)
modelCoefficients <- function(x) {
    assert_that(!is.vector(x), msg = "not a valid model object")
    assert_that(!is.null(x$model), msg = "not a valid model object")
    # obtain model and extract member functions
    model <- canonicalModel_(attr(x, "model"))
    coefficients_ <- model$coefficients
    x <- coefficients_(x)
    ret <- x[, "Estimate"]
    names(ret) <- rownames(x)
    return(ret)
}

#' Obtain model AIC
#'
#' @param x fitted model
#' @return AIC for model
#' @export
#' @examples
#' modelAIC(some_model)
modelAIC <- function(x) {
    assert_that(!is.vector(x), msg = "not a valid model object")
    assert_that(!is.null(x$model), msg = "not a valid model object")
    # obtain model and extract member functions
    model <- canonicalModel_(attr(x, "model"))
    return(model$aic(x))
}

#' Obtain model input data
#'
#' @param x fitted model
#' @return  Input data for the fitted model
#' @export
#' @examples
#' x <- modelData(some_model)
modelData <- function(x) {
    assert_that(!is.vector(x), msg = "not a valid model object")
    assert_that(!is.null(x$model), msg = "not a valid model object")
    # obtain model and extract member functions
    model <- canonicalModel_(attr(x, "model"))
    return(model$data(x))
}

#' Obtain p-values for terms in model formula
#'
#' @param x fitted model
#' @return  Pvalue for the terms in the fitted model
#' @export
#' @examples
#' modelTermPvalues(some_model)
modelTermPvalues <- function(x) {
    assert_that(!is.vector(x), msg = "not a valid model object")
    assert_that(!is.null(x$model), msg = "not a valid model object")
    model <- canonicalModel_(attr(x, "model"))
    return(model$pterm(x))
}

#' Obtain model p-value
#'
#' @param x fitted model
#' @return Pvalue for the model
#' @export
#' @examples
#' modelModelPvalue(some_model)
modelModelPvalue <- function(x) {
    assert_that(!is.vector(x), msg = "not a valid model object")
    assert_that(!is.null(x$model), msg = "not a valid model object")
    model <- canonicalModel_(attr(x, "model"))
    return(model$pmodel(x))
}

#' Obtain model name
#'
#' @param x fitted model
#' @return model name
#' @export
#' @examples
#' modelModelName(some_model)
modelModelName <- function(x) {
    assert_that(!is.vector(x), msg = "not a valid model object")
    assert_that(!is.null(x$model), msg = "not a valid model object")
    return(attr(x, "model"))
}


#' Decifer a 'model parameter' and run appropriate \code{glm_...} function.
#'
#' Return canonical model from model type string, function of object. Returns
#' a model as returned by \code{glm_gaussian()} and others, based on a string,
#' function or model type object (i.e. \code{"glm_gaussian"}, \code{glm_gaussian} or
#' \code{glm_gaussian()}).
#'
#' @param model string, function or object representing a model type.
#' @return model type object
canonicalModel_ <- function(model) {
    # figure out the type of model.
    assert_that(!is.null(model))
    if (is.function(model)) {
        # function was supplied: run the function
        x <- model()
    } else if (is.character(model)) {
        # character vector was supplied: try to obtain function of specified name
        assert_that(1 == length(model))
        model <- get(model, mode = "function", envir = parent.frame())
        x <- model()
    } else {
        # assume that what we got was the result of model function call
        x <- model
    }
    # by now x should be a structure with a "model" entry
    assert_that(!is.null(x$model))
    return(x)
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#'  geneVari Makes a list of gene names to be used in the runModels function
#'
#' This function defines the boudnaries of mRNA vs miRNAs of interest to be analysed by the runModels function
#' @param Combined the combined file for mRNA and selected miRNAs output of combiner function
#' @param miRNA_select The vector of selected miRNA/s
#' @return A vector of characters with defined mRNA dimensions
#' @export
#' @keywords dimensions
#' @examples
#' x <- geneVari(Combine, "ebv-mir-bart9-5p")
geneVari <- function(Combined, miRNA_select) {
    # geneVari1 <- colnames(Combined[, 1:(ncol(Combined) - length(miRNA_select))])
    geneVari1 <- setdiff(colnames(Combined), miRNA_select)
    return(geneVari1)
}

# No need to download a second time if downloaded once we use memoization
.onLoad <- function(libname, pkgname) {
    # setup up memoization for expensive functions (if R.cache available)
    if (requireNamespace("R.cache", quietly = TRUE)) {
        assign("getInputSpecies_", R.cache::addMemoization(getInputSpecies_), envir = parent.env(environment()))
        assign("downloadMirandaFile_", R.cache::addMemoization(downloadMirandaFile_), envir = parent.env(environment()))
        assert_that(!is.null(getInputSpecies_) && is.function(getInputSpecies_)) # if NULL, memoization failed
        assert_that(!is.null(downloadMirandaFile_) && is.function(downloadMirandaFile_)) # if NULL, memoization failed
    }
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#' finInterResult miRNA and mRNA interrelation in two-time points results in a dataframe.
#'
#' This function uses the output of one2OneRnaMiRNA and returns a sampled from orig file
#' interrelation dataframe depending on user sampling selection.
#' @param results Results from mirandaIntersectInter
#' @return miRNA mRNA interelation dataframe
#' @export
#' @keywords Results dataframe
#' @examples
#' x <- finInterResult(results)
finInterResult <- function(results) {
    final_results <- data.frame(
        mRNA = rownames(results$mrna),
        miRNA = rownames(results$mirna),
        FC_mRNA = results$mrna$FC1,
        FC_miRNA = results$mirna$FC1,
        pvalue = results$corrs$pvalue
    )
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#' miRandaIntersect Looks for Intersection of Significant output results with miRanda Results from getInputSpeciesDF
#' function
#'
#' Compares and looks for intersection if significant output results with miRanda Results from getInputSpeciesDF and outputs a final
#' filtered output for only those pairs of miRNA and mRNA which have actually been predicted to be targets
#' in miRanda file
#' function
#' @param sig_corrs correlation matrix, produced by threshSig.
#' @param corrS vector of correlations/differences, from the sampCorRnaMirna function.
#' @param mRNA mRNA FC matrix.
#' @param miRNA miRNA FC matrix.
#' @param getInputSpeciesDF miranda data, produced by getInputSpecies.
#' @return An object containing data.frames of significant mRNA, miRNA
#'             and correlation matrix filtered by miRanda input.
#' @export
#' @keywords Significance, Threshold, intersect
#' @examples
#' x <- miRandaIntersect(sig_InterR, outs2, mRNA_fc, miRNA_fc, miRandaM)
miRandaIntersect <- function(sig_corrs, corrS, mRNA, miRNA, getInputSpeciesDF) {
    result_corrs <- dplyr::inner_join(sig_corrs, getInputSpeciesDF, by = c("V1", "V2"))
    result_mrna <- mRNA[result_corrs$V2, , drop = FALSE]
    result_mirna <- miRNA[result_corrs$V1, , drop = FALSE]
    # calculate "p-values" for correlations. (TODO check this.)
    # count how many values in corrS are > each correlation and calculate 1 - "percentage".
    pvalueiods <- 1 - sapply(result_corrs$value, function(x) {
        length(which(corrS > x))
    }) / length(corrS)

    # add p-value for result_corrs df
    result_corrs$pvalue <- pvalueiods
    return(list(mirna = result_mirna, mrna = result_mrna, corrs = result_corrs))
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#' @importFrom pscl zeroinfl zeroinfl.control
#' @importFrom assertthat assert_that
#' @importFrom purrr is_formula
#' @importFrom stats formula AIC
NULL


#' Model functions for zero inflated model using either Poisson or Negative Binomial distributions.
#'
#' Implements standardaized functions to fit the zero inflated model with
#' Poisson or Negative Binomial distribution, and to obtain coefficients,
#' pvalues, etc.
#'
#' @param dist either 'poisson' or 'negbin'
#' @return structure containing functions \code{fit}, \code{coefficients},
#'         \code{aic}, \code{data}, \code{pterm}, \code{pmodel}, and a
#'         character string "glm_zeroinfl" in \code{model}.
#' @export
#' @examples
#' x <- glm_zeroinfl("negbin")
glm_zeroinfl <- function(dist = "poisson") { # poisson, negbin
    assert_that(dist %in% c("poisson", "negbin"))
    model_name <- sprintf("glm_zeroinfl_%s", dist)

    transform_ <- function(x, data, scale) {
        mf <- modelframe_(x, data) # extract model data.frame
        mf[, 1] <- as.integer(round(mf[, 1] * scale)) # scale and round response variable
        attr(mf, "scale") <- scale # add 'scale' as attribute to data.frame
        return(mf)
    }

    fit_ <- function(x, data) {
        if (is_formula(x)) {
            x <- as.character.formula(x)
        }
        if (!grepl("\\|", x)) {
            x <- paste(x, "|", "1")
        }
        tryCatch(
            {
                g <- zeroinfl(formula(x),
                    data = data, dist = dist,
                    control = zeroinfl.control(maxit = 100000)
                )
                if (!is.null(g)) {
                    attr(g, "model") <- model_name
                }
                return(g)
            },
            warning = function(e, ...) {
                #        warning(e)
                return(NULL)
            },
            error = function(e, ...) {
                #        warning(e)
                return(NULL)
            }
        )
    }

    # fit
    fit <- function(x, data, scale = 1, ...) {
        data <- transform_(x, data, scale)
        ret <- fit_(x, data)
        if (!is.null(ret)) {
            ret$data <- data
        }
        return(ret)
    }

    pterm <- function(model) {
        nm <- termlabels_(model$formula)
        ret <- coefficients_(model)[, "Pr(>|z|)", drop = TRUE]
        names(ret) <- nm
        return(ret)
    }

    coefficients_ <- function(model) {
        nm <- termlabels_(model$formula)
        return(as.data.frame(summary(model)$coefficients$count)[nm, ])
    }

    aic_ <- function(model) {
        return(AIC(model))
    }

    structure(list(
        fit = fit, coefficients = coefficients_, aic = aic_, data = data_,
        pterm = pterm, pmodel = pmodel_, model = model_name
    ))
}

#' alias for glm_zeroinfl("poisson")
#' @param ... passed to \code{glm_zeroinfl}
#' @return structure containing functions \code{fit}, \code{coefficients},
#'         \code{aic}, \code{data}, \code{pterm}, \code{pmodel}, and a
#'         character string "glm_zeroinfl" in \code{model}.
#' @export
#' @examples
#' x <- glm_zeroinfl_poisson()
glm_zeroinfl_poisson <- function(...) glm_zeroinfl("poisson")

#' alias for glm_zeroinfl("negbin")
#' @param ... passed to \code{glm_zeroinfl}
#' @return structure containing functions \code{fit}, \code{coefficients},
#'         \code{aic}, \code{data}, \code{pterm}, \code{pmodel}, and a
#'         character string "glm_zeroinfl" in \code{model}.
#' @export
#' @examples
#' x <- glm_zeroinfl_negbin()
glm_zeroinfl_negbin <- function(...) glm_zeroinfl("negbin")

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#' @importFrom stats density quantile
#' @importFrom graphics lines abline
NULL

#' mirRnaDensityCor for miRTarRNASeq miRNA and mRNA correlation real data versus sampled data
#'
#' This function draws density plots for miRNA and mRNA correlation while
#' comparing real data vs sampled data. It mainly illustrates the where the lower %5 (sig)
#' relationships lie.
#' @param corr0 data.frame results of corMirnaRna function.
#' @param corrS data.frame results from the sampCorRnaMirna function.
#' @param pvalue The p value threshold to be used on the data density plot default is 0.05.
#' @return Density plot
#' @export
#' @keywords Density plot
#' @examples
#' x <- mirRnaDensityCor(corr_0, outs, pvalue = 0.05)
mirRnaDensityCor <- function(corr0, corrS, pvalue = 0.05) {
    dens_corr_0 <- density(corr0$value)
    dens_outs <- density(corrS) # this is the distribution for correlations for shuffled data
    mx_y <- max(dens_corr_0$y, dens_outs$y)

    plot(
        dens_outs,
        xlim = c(-1.5, 1.5),
        ylim = c(0, mx_y),
        col = "grey80",
        lwd = 2
    )
    lines(dens_corr_0, col = "red", lwd = 2) # this is what we got for our original data
    abline(
        v = c(-1, 1),
        col = "grey90",
        lty = 2
    ) # indicate -1 and 1 in the plot.
    # find a threshold based on the shuffled data
    threshold <- quantile(corrS, pvalue) # 5% if else user can define
    abline(v = threshold, col = "blue", lty = 2) # add theshold line to density plots
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#' @importFrom grDevices colorRampPalette
#' @importFrom graphics par abline
#' @importFrom stats density
NULL

#' drawInterPlots for finInterResult miRNA and mRNA Interrelation real data
#'
#' This function draws miRNA, mRNA density plots for miRNA and mRNA Interrelation while
#' comparing in addition to overall FC_miRNA and FC_mRNA plots from the finInterResult dataframe function.
#' @param mrna mRNA results of twoTimePoint function.
#' @param mirna miRNA results of twoTimePoint function.
#' @param final_results finInterResult miRNA and mRNA interrelation in two timepoints  results in a dataframe.
#' @return par plots
#' @export
#' @keywords plot
#' @examples
#' x <- drawInterPlots(mRNA_fc2, miRNA_fc2, final_results)
drawInterPlots <- function(mrna, mirna, final_results) {
    par(mfrow = c(2, 2))

    # plot density of mRNA FCs
    plot(density(mrna$FC1))
    abline(v = 0, col = "grey80", lty = 2)

    # plot density of miRNA FCs
    plot(density(mirna$FC1))
    abline(v = 0, col = "grey80", lty = 2)

    # plot final results (i.e. p < 0.05) mRNA vs miRNA FCs
    # FC_mRNA and FC_miRNA are coloumns of final results
    with(final_results, plot(FC_mRNA, FC_miRNA))
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#' @importFrom stats cor
#' @importFrom dplyr `%>%`
NULL

## quiet concerns of R CMD check regarding unbound global variables (in dplyr::filter() calls)
if (getRversion() >= "2.15.1") {
    utils::globalVariables(c("V1", "V2", "V3", "V4", "V5", "V6"))
}

#' Return Miranda data for a given species.
#'
#' Reads Miranda file for a given speicies and returns it as
#' a data.frame, thresholded by percent identity.
#' Header options are Score (threshold), Energy-Kcal/Mol(energy),
#' Subject-IdentityPercent(targetIden), Query-IdentityPercent (mirnaIden)
#' @param selection Species (species selection are either for mature miRNA species "Human1","Mouse",
#'  "C.elegans", "Epstein_Barr", "Epstein_Barr_Human",
#'  "Drosophila","Kaposi_Sarcoma", "KSHV_Human",
#'  "Cytomegalovirus","CMV_Human")
#' @param threshold miRanda score threshold default 60
#' @param energy miRanda folding energy threshold default NULL
#' @param targetIden miRanda target identity score default NULL
#' @param mirnaIden miRanda mirna identity score default NULL
#' @return data.frame with Miranda data.
#' @keywords miranda, species
#' @export
#' @examples
#' x <- getInputSpecies("Epstein_Barr", threshold = 60) # Default is threshold 60
getInputSpecies <- function(selection, threshold = 60, energy = NULL, targetIden = NULL, mirnaIden = NULL) {
    return(getInputSpecies_(
        selection = selection, threshold = threshold,
        energy = energy, targetIden = targetIden,
        mirnaIden = mirnaIden
    ))
}

# use internal function here so we can R.cache it without changing how it looks like for the user.
getInputSpecies_ <- function(selection, threshold = 60, energy = NULL, targetIden = NULL, mirnaIden = NULL) {
    if (selection == "Human1") {
        ret <- downloadMirandaFile("https://zenodo.org/record/4615670/files/Human_miRanda.txt.gz")
    } else if (selection == "Mouse") {
        ret <- downloadMirandaFile("https://zenodo.org/record/4615670/files/Mouse_miRanda.txt.gz")
    } else if (selection == "Epstein_Barr") {
        ret <- importMirandaFile("Epstein_Barr_miRanda.txt.gz")
    } else if (selection == "C.elegans") {
        ret <- downloadMirandaFile("https://zenodo.org/record/4615670/files/C_elegans_miRanda.txt.gz")
    } else if (selection == "Drosophila") {
        ret <- downloadMirandaFile("https://zenodo.org/record/4615670/files/Drosophila_miRanda.txt.gz")
    } else if (selection == "Cytomegalovirus") {
        ret <- downloadMirandaFile("https://zenodo.org/record/4898541/files/CMV_miRanda.txt.gz")
    } else if (selection == "Kaposi_Sarcoma") {
        ret <- downloadMirandaFile("https://zenodo.org/record/4898541/files/Kaposi_miRanda.txt.gz")
    } else if (selection == "Epstein_Barr_Human") {
        ret <- downloadMirandaFile("EBV_Human_miRanda.txt.gz")
    } else if (selection == "CMV_Human") {
        ret <- downloadMirandaFile("https://zenodo.org/record/4898541/files/CMV_Human_miRanda.txt.gz")
    } else if (selection == "KSHV_Human") {
        ret <- downloadMirandaFile("https://zenodo.org/record/4898541/files/KSHV_Human_miRanda.txt.gz")
    }
    ret <- dplyr::filter(ret, V3 >= threshold)
    if (!is.null(energy)) {
        ret <- dplyr::filter(ret, V4 <= energy)
    }
    if (!is.null(targetIden)) {
        ret <- dplyr::filter(ret, V5 >= targetIden)
    }
    if (!is.null(mirnaIden)) {
        ret <- dplyr::filter(ret, V6 >= mirnaIden)
    }
    ret <- ret %>% dplyr::select(V1, V2, V3, V4, V5, V6)
    # ret <- unique(ret)
    return(ret)
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#'  miRanComp comparison of mRNAs present with miRanda file targets
#'
#' This function generates a dataframe consisting of mRNA or miRNAs present in
#'  miRanda generated file using the miRTarRNASeq:::getInputSpecies() function
#' @param miRNA Matrix or data.frame miRNA/RNA file or transformed diff expression file (generated
#' using TZtranz)
#' @param miRanda A dataframe of miRanda file with miRNA$V1 and miRNA targets miRNA$V2
#' @return An miRNA expression dataframe which includes only Genes/Targets present in miRanda file
#' @export
#' @keywords miRanda expression Intersection
#' @examples
#' x <- miRanComp(miRNA, miRanda)
miRanComp <- function(miRNA, miRanda) {
    subset <- intersect(rownames(miRNA), miRanda$V2)
    kept <- miRNA[subset, ]
    return(kept)
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#' @importFrom stats poisson glm
NULL

#' Model functions for GLM with Poisson model.
#'
#' Implements standardaized functions to fit the glm with
#' Poisson family and to obtain coefficients, pvalues, etc.
#'
#' @return structure containing functions \code{fit}, \code{coefficients},
#'         \code{aic}, \code{data}, \code{pterm}, \code{pmodel}, and a
#'         character string "glm_poisson" in \code{model}.
#' @export
#' @examples
#' x <- glm_poisson()
glm_poisson <- function() {
    model_name <- "glm_poisson"

    transform_ <- function(x, data, scale) {
        mf <- modelframe_(x, data) # extract model data.frame
        mf[, 1] <- as.integer(round(mf[, 1] * scale)) # scale and round response variable
        attr(mf, "scale") <- scale # add 'scale' as attribute to data.frame
        return(mf)
    }

    fit_ <- function(x, data) {
        tryCatch(
            {
                g <- glm(x, data = data, family = poisson())
                if (!is.null(g)) {
                    attr(g, "model") <- model_name
                }
                return(g)
            },
            warning = function(e, ...) {
                #        warning(e)
                return(NULL)
            },
            error = function(e, ...) {
                #        warning(e)
                return(NULL)
            }
        )
    }

    # fit
    fit <- function(x, data, scale = 1, ...) {
        data <- transform_(x, data, scale)
        return(fit_(x, data))
    }

    pterm <- function(model) {
        nm <- termlabels_(model$formula)
        ret <- coefficients_(model)[, "Pr(>|z|)", drop = TRUE]
        names(ret) <- nm
        return(ret)
    }

    structure(list(
        fit = fit, coefficients = coefficients_, aic = aic_, data = data_,
        pterm = pterm, pmodel = pmodel_, model = model_name
    ))
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#' @importFrom dplyr filter
NULL

## quiet concerns of R CMD check regarding unbound global variables (in dplyr::filter() calls)
if (getRversion() >= "2.15.1") {
    utils::globalVariables(c("pvalue"))
}

## Inputs mRNA or miRNA dif expression file

#' one2OneRnaMiRNA correlation for miRNA and mRNA  using differntial expression
#' fold change and if/when available p-value
#'
#' This function inputs accept a list of dataframes and returns an obj with
#' two dataframes called FC and p-value. FC with rownames == genes and columns are FC1, 2, 3, ...
#' (with fold-changes) - P-value with rownames == genes and columns are P1, 2, 3, ... (with p-values)
#' both data.frames have the same order dimensions.
#' @param files a list of dataframes either miRNAs or mRNAs from various time points.
#' @param gene_colname Default is a vector character of length 1 "Gene" user can alter if they choose
#' This column contains the gene names.
#' @param fc_colname Default "FC" is coloumn name for fold changes user can alter if they choose.
#' @param pval_colname Default is "pvalue" column name for p-values (in input).
#' @param pthreshold P-value threshold.
#' @return Correlation dataframe
#' @export
#' @keywords correlation
#' @examples
#' \donttest{
#' x <- one2OneRnaMiRNA(files)
#' }
one2OneRnaMiRNA <- function(files, gene_colname = "Gene", fc_colname = "FC",
                            pval_colname = "pvalue", pthreshold = NULL) {


    # asser file......
    # assert is.character(gene_colname) && length(gene_colname) == 1
    # assert is.character(fc_colname) && length(fc_colname) == 1
    # assert (is.numeric(pthreshold) && length(pthreshold) == 1) || is.null(pthreshold)
    # assert ... gene col is.character! not factor.

    # do some mininal cleaning here...
    files <- lapply(files, function(x) {
        x <- as.data.frame(x)
        x[, gene_colname] <- as.character(x[, gene_colname]) # account for users...
        return(x)
    })

    # grab fold changes and gene names from each data.frame supplied
    foldchanges <- lapply(files, function(x) {
        ret <- x[, c(gene_colname, fc_colname)]
        rownames(ret) <- ret[, gene_colname, drop = TRUE]
        return(ret)
    })

    if (!is.null(pthreshold)) {
        # grab p-values and gene names from each data.frame supplied
        pvalues <- lapply(files, function(x) {
            ret <- x[, c(gene_colname, pval_colname)]
            rownames(ret) <- ret[, gene_colname, drop = TRUE]
            return(ret)
        })

        # produce set of genes significant in any of the gene lists
        # all genes concatenated in one long unique list
        siggenes <- unique(unlist(sapply(pvalues, function(x) {
            ret <- filter(x, pvalue < pthreshold)[, gene_colname, drop = TRUE]
            return(ret)
        }))) # for some reason sapply doesn't simplify here... hence the 'unlist'
    } else {
        # get set of genes regardless of p-value
        pvalues <- NULL
        siggenes <- unique(unlist(sapply(foldchanges, function(x) {
            x[, gene_colname, drop = TRUE] # we're producing a long vector here...
        })))
    }

    # filter siggenes for gene names present in all input lists!
    for (x in foldchanges) {
        siggenes <- intersect(siggenes, x[, gene_colname, drop = TRUE]) # we're intersecting _vectors_ here
    }

    # filter and reorder FCs by sig genes
    foldchanges <- lapply(foldchanges, function(x) {
        return(x[siggenes, fc_colname, drop = FALSE])
    })

    # make one data frame with _just_ the FC columns.
    foldchanges <- do.call(cbind.data.frame, foldchanges)
    colnames(foldchanges) <- paste("FC", seq_len(ncol(foldchanges)), sep = "") # cleanup column names

    if (!is.null(pthreshold)) {
        # filter and reorder pvalues by sig genes
        pvalues <- lapply(pvalues, function(x) {
            return(x[siggenes, pval_colname, drop = FALSE])
        })

        # make one data frame with _just_ the pvalue columns.
        pvalues <- do.call(cbind.data.frame, pvalues)
        colnames(pvalues) <- paste("P", seq_len(ncol(pvalues)), sep = "") # cleanup column names
    }

    # now 'foldchanges' and 'pvalues' are _data.frame_ with _just_ FC/P columns.
    return(list(foldchanges = foldchanges, pvalues = pvalues))
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#' @importFrom stats p.adjust
NULL

#' fdrSig Ruturns FDR significant miRNA/mRNA predictions
#'
#' This function performs FDR correction on the p_values generated by the runModels function list.
#' @param RMObj The output of runModels
#' @param value The FDR value default is 0.1
#' @param method The p-value adjustment method default is fdr. It could be either of the following "holm", "hochberg",
#' "hommel","bonferroni", "BH", "BY", or "fdr".
#' @return A list of FDR corrected p vlaues, annova, and significance for each gene and the miRNA/s of interest
#' @export
#' @keywords p_adjust, correction
#' @examples
#' models <- runModels(Combine, geneVariant, "ebv-mir-bart9-5p")
#' x <- fdrSig(models, value = 0.1, method = "fdr")
fdrSig <- function(RMObj, value = 0.05, method = "fdr") {
    p_adjusted <- apply(RMObj$pvalues, 2, p.adjust, method = method)
    is_significant <- apply(p_adjusted, 1, function(x) any(is.finite(x) & x < value))
    sig_models <- RMObj$all_models[is_significant]
    sig_genes <- RMObj$genes[is_significant]
    sig_pvalues <- RMObj$pvalues[is_significant, , drop = FALSE]
    sig_p_adjusted <- p_adjusted[is_significant, , drop = FALSE]
    # add calculated values to original object and return
    RMObj$p_adjusted <- p_adjusted
    RMObj$is_significant <- is_significant
    RMObj$sig_models <- sig_models
    RMObj$sig_genes <- sig_genes
    RMObj$sig_pvalues <- sig_pvalues
    RMObj$sig_p_adjusted <- sig_p_adjusted
    return(RMObj)
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#' twoTimePoint miRNA and mRNA interrelation in two timepoints
#'
#' This function uses the output of one2OneRnaMiRNA and returns a sampled from original file
#' interrelation dataframe depending on user sampling selection.
#' @param mRNA mRNA file generated from fold changes (FC) obj of the one2OneRnaMiRNA.
#' @param miRNA miRNA file generated from fold changes (FC) obj of the one2OneRnaMiRNA.
#' @return miRNA mRNA interrelation dataframe
#' @export
#' @keywords mRNA miRNA interrelation
#' @examples
#' \donttest{
#' x <- twoTimePoint(mRNA_fc2, miRNA_fc2)
#' }
twoTimePoint <- function(mRNA, miRNA) {
    assertthat::assert_that(ncol(mRNA) == 1 && ncol(miRNA) == 1)
    cc <- matrix(0, nrow = nrow(miRNA), ncol = nrow(mRNA))
    rownames(cc) <- rownames(miRNA)
    colnames(cc) <- rownames(mRNA)
    for (i in seq_len(nrow(miRNA))) {
        for (j in seq_len(nrow(mRNA))) {
            cc[i, j] <- abs(miRNA[i, 1] - mRNA[j, 1])
        }
    }
    mmycc <- reshape2::melt(cc)
    names(mmycc) <- c("V1", "V2", "value")
    mmycc$V1 <- as.character(mmycc$V1)
    mmycc$V2 <- as.character(mmycc$V2)
    return(as.data.frame(mmycc))
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#' @importFrom purrr is_formula
#' @importFrom stats terms formula model.frame deviance pchisq df.residual
NULL

coefficients_ <- function(model) {
    nm <- attr(terms(model$formula), "term.labels")
    return(as.data.frame(summary(model)$coefficients)[nm, ])
}

pmodel_ <- function(model) {
    ret <- pchisq(model$null.deviance - deviance(model), model$df.null - df.residual(model), lower.tail = FALSE)
    # ret <- pchisq(model$deviance, model$df.residual, lower.tail = FALSE)
    return(ret)
}

aic_ <- function(model) {
    return(model$aic)
}

data_ <- function(model) {
    return(model$data)
}

modelframe_ <- function(x, data) {
    if (is_formula(x)) {
        x <- as.character.formula(x)
    }
    x <- formula(gsub("\\|", "+", x))
    model.frame(x, data) # extract model data.frame
}

termlabels_ <- function(x) {
    f <- formula(gsub("\\|.*$", "", as.character.formula(x)))
    nm <- attr(terms(f), "term.labels")
}

# notes on model evaluation
# https://www2.stat.duke.edu/courses/Fall17/sta521/knitr/Lec-9-Poisson-Checks/predictive-checking.pdf
# https://stats.stackexchange.com/questions/129958/glm-in-r-which-pvalue-represents-the-goodness-of-fit-of-entire-model
# http://thestatsgeek.com/2014/04/26/deviance-goodness-of-fit-test-for-poisson-regression/
# http://users.stat.umn.edu/~helwig/notes/msd-Notes.pdf
# https://www.scribbr.com/statistics/akaike-information-criterion/

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#' @importFrom stats quantile
#' @importFrom caTools combs
NULL

#' runAllMirnaModels runModel for all miRNAs
#'
#' This function runs the "runModel" function for all miRNAs and mRNA combinations of two and returns a
#' list with significant genes and FDR models
#' @param mirnas vector of unique miRNAs under investigation.
#' @param DiffExpmRNA deferentially/expressed mRNAs expression file.
#' @param DiffExpmiRNA deferentially/expressed miRNAs expression file.
#' @param miranda_data getInputSpecies output file ( use low filters).
#' @param prob user defined ratio for miRanda distribution for miRanda score selection default is 0.75.
#' @param fdr_cutoff cutoff for FDR selection default is 0.1.
#' @param method finInterResult miRNA and mRNA interrelation in two time points  results in a dataframe.
#' @param cutoff P-value cutoff of the model.
#' @param all_coeff if true only models with all negative coefficients will be selected if false at least one
#' negative coefficient should be in the model; default is TRUE.
#' @param family Default is glm_poisson(), for zero inflated negative binomial NB option use glm_zeroinfl(dist="negbin").
#' @param mode model mode, default is Null, can be changed to "multi" and "inter".
#' @param scale  if normalized data (FPKM,RPKM,TPM,CPM), scale to 10 etc., however the higher you go on
#' #scale the less accuracy your p-value estimate will be.
#' @return List of run models
#' @export
#' @keywords runAllMirnaModels all_miRNAs
#' @examples
#' mirnas <- c("ebv-mir-bart9-5p", "ebv-mir-bart6-3p")
#' x <- runAllMirnaModels(mirnas, mRNA, miRNA, miRanda,
#'     prob = 0.90, fdr_cutoff = 0.1, method = "fdr",
#'     all_coeff = TRUE, mode = "multi",
#'     family = glm_poisson(), scale = 100
#' )
runAllMirnaModels <- function(mirnas, DiffExpmRNA, DiffExpmiRNA, miranda_data,
                              prob = 0.75, fdr_cutoff = 0.1, method = "fdr", cutoff = 0.05,
                              all_coeff = FALSE, mode = NULL, family = glm_poisson(), scale = 1) {
    assertthat::assert_that(is.character(mirnas))

    # make unique
    unique_mirnas <- unique(mirnas)

    # generate combinations of 2 miRNAs
    if (!is.null(mode)) {
        # produces list of pair of mirnas, e.g.:
        #
        # ...
        # [[120]]
        # [1] "ebv-mir-bart11-3p" "ebv-mir-bart18-3p"
        #
        # [[121]]
        # [1] "ebv-mir-bart11-3p" "ebv-mir-bart18-5p"
        #
        # [[122]]
        # [1] "ebv-mir-bart11-3p" "ebv-mir-bart19-3p"
        # ...
        unique_mirnas <- caTools::combs(unique_mirnas, 2)
        unique_mirnas <- lapply(seq_len(nrow(unique_mirnas)), function(x) unique_mirnas[x, ])
    }

    # run models
    count <- 0
    ret <- lapply(unique_mirnas, function(mirna) {
        count <<- count + 1
        cat(sprintf("%d: %s\n", count, paste(mirna, collapse = ", ")))

        # For Determining Prob it needs to percent
        valmMedia <- quantile(miranda_data$V3, probs = prob)[1]
        # MirandaParsingOfmyRNA
        DiffExpmRNASub <- miRanComp(DiffExpmRNA, miranda_data)

        # everything <- lapply(uni_miRanda, function(mirna) {
        # combiner
        Combine <- combiner(DiffExpmRNASub, DiffExpmiRNA, mirna)

        # GeneVari
        geneVariant <- geneVari(Combine, mirna)

        MRun <- runModels(Combine, geneVariant, mirna, mode = mode, family = family, scale = scale, cutoff = cutoff)

        # #Bonferroni sig Genes
        FDRModel <- fdrSig(MRun, value = fdr_cutoff, method = method)

        if (length(FDRModel$sig_models) > 0) {
            # look for negative coeffs; "-1" to exclude intercept while checking...#If true all if false any
            if (all_coeff) {
                valid_models <- sapply(FDRModel$sig_models, function(m) all(modelCoefficients(m) < 0))
            } else {
                valid_models <- sapply(FDRModel$sig_models, function(m) any(modelCoefficients(m) < 0))
            }
            # subset fdrmodel's things
            FDRModel$sig_models <- FDRModel$sig_models[valid_models]
            FDRModel$sig_pvalues <- FDRModel$sig_pvalues[valid_models, , drop = FALSE]
            FDRModel$sig_p_adjusted <- FDRModel$sig_p_adjusted[valid_models, , drop = FALSE]
            FDRModel$sig_genes <- FDRModel$sig_genes[valid_models]
        }
        # make siggenes data.frame
        SigFDRGenes <- data.frame(cbind(FDRModel$sig_pvalues, FDRModel$sig_p_adjusted))

        names(SigFDRGenes) <- c("P Value", "FDR")

        return(list(SigFDRGenes = SigFDRGenes, FDRModel = FDRModel))
    })
    names(ret) <- sapply(unique_mirnas, paste, collapse = " and ")
    return(ret)
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

# convert formula to character (same as private formula.tools:::as.character.formula)
as.character.formula <- function(x, ...) {
    # see formula.tools:::as.character.formula
    form <- paste(deparse(x), collapse = " ")
    form <- gsub("\\s+", " ", form, perl = FALSE)
    return(form)
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#' sampCorRnaMirna sampling for correlation for miRNA and mRNA
#'
#' This function uses the output of one2OneRnaMiRNA and returns a sampled from original file
#' correlation dataframe depending on user sampling selection.
#' @param mRNA mRNA file generated from fold changes (FC) obj of the one2OneRnaMiRNA.
#' @param miRNA miRNA file generated from fold changes (FC) obj of the one2OneRnaMiRNA.
#' @param method Default is "pearson" else use "kendall" or "spearman".
#' @param Shrounds number of shuffling over the FC data, default is 100.
#' @param Srounds number of sampling from the shuffled data, default is 1000.
#' @return Correlation data frame
#' @export
#' @keywords sampling, sampling, correlation, shuffling
#' @examples
#' x <- sampCorRnaMirna(mRNA_fc, miRNA_fc, method = "pearson", Shrounds = 10, Srounds = 10)
sampCorRnaMirna <- function(mRNA, miRNA, method = "pearson", Shrounds = 100, Srounds = 1000) {
    outs <- c()
    for (i in seq_len(Shrounds)) {
        print(i)
        shuffled_mrna <- mRNA
        shuffled_mirna <- miRNA
        for (col in seq_len(ncol(shuffled_mrna))) {
            shuffled_mrna[, col] <- sample(shuffled_mrna[, col]) # this shuffles all values in column _col_
            shuffled_mirna[, col] <- sample(shuffled_mirna[, col]) # this shuffles all values in column _col_
        }
        cc <- corMirnaRna(shuffled_mrna, shuffled_mirna, method = method) # run correlation on shuffled data
        outs <- c(outs, sample(cc$value, Srounds, replace = TRUE)) # take a sample of the correlations and add to _outs_
    }
    return(outs)
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#' Plot residuals
#'
#' Plot residuals description
#' @param model linear model
#' @return does not return value
#' @export
#' @keywords plot, residuals, model
#' @examples
#' plotResiduals(lm(x ~ y, data = data.frame(x = runif(10), y = runif(10))))
plotResiduals <- function(model) {
    plot(model, 1)
    return(invisible(NULL))
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#' @importFrom graphics par abline points mtext
#' @importFrom stats pf lm
NULL

#' Plot model
#'
#' Plot 2D description
#' @param model linear model
#' @return does not return value
#' @export
#' @keywords plot, 2d, model
#' @examples
#' plotFit(lm(x ~ y, data = data.frame(x = runif(10), y = runif(10))))
plotFit <- function(model) {
    m <- model$model
    m$Fitted <- model$fitted.values
    last <- ncol(m)
    names(m)[last] <- "Fitted values\nlm(model_formula)"
    par(mar = c(5, 5, 4, 2) + 0.1)
    plot(m[, c(1, last)],
        type = "n", main = as.character.formula(model$terms),
        cex.main = 1
    )
    abline(lm(m[, last] ~ m[, 1]), lwd = 2, col = "red")
    points(m[, c(1, last)])

    k <- length(model$coefficients) - 1 # Number of mirnas in model!
    n <- length(model$fitted.values)
    rs <- summary(model)$r.squared
    Pvalue <- pf((rs / k) / ((1 - rs) / (n - 1 - k)), k, n - 1 - k, lower.tail = FALSE) # p value for R-squared

    mtext(sprintf("P value: %.2g", Pvalue), line = .74)
    mtext(sprintf("R-squared: %.2f", summary(model)$r.squared), line = .01)
    return(invisible(NULL))
}

#' ## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020
#'
#'  #' rsquRes returns r matrix of correlation
#'  #'
#'  #' This function parses R correlation value of the from the list of significant FDR adjusted models with
#'  #' negative/positive correlations.
#'  #' @param FDRSigList list of FDR adjusted models with negative/positive correlations.
#'  #' @return matrix of correlation
#'  #' @export
#'  #' @keywords R correlation
#'  #' @examples
#'  #' \donttest{
#'  #' x <- rsquRes(FDRSigList)
#'  #' }
#'  #'
#' rsquRes <- function(FDRSigList) {
#'  # for all mirnas, for all the models get the r-squares
#'  rsquares <- lapply(FDRSigList, function(x) sapply(x[["all_models"]], modelRsquared))
#'
#' # make rownames and colnames for the matrix we want to make
#' colnames <- sort(unique(unlist(sapply(rsquares, names))))
#' rownames <- names(rsquares)
#' # make matrix with zeros
#' m <- matrix(0.0, nrow = length(rownames), ncol = length(colnames))
#'  # set colnames and rownames
#' rownames(m) <- rownames
#' colnames(m) <- colnames
#' # for all mirnas, put the rsqaures in the right places (by name)
#'  for (row in rownames) {
#'  m[row, names(rsquares[[row]])] <- rsquares[[row]]
#'  }
#'
#'  m1 <- -sqrt(m)
#'  return(m1)
#' }

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#' @importFrom reshape2 dcast
#' @importFrom dplyr `%>%`
#' @importFrom pheatmap pheatmap
#' @importFrom viridis inferno
NULL

## quiet concerns of R CMD check regarding unbound global variables (in dplyr::filter() calls)
if (getRversion() >= "2.15.1") {
    utils::globalVariables(c("V1"))
}

#' mirRnaHeatmap pheatmap for miRTarRNASeq miRNA and mRNA correlation
#'
#' This function draws pheatmaps for miRNA and mRNA correlation while
#' using default and pheatmap for all other parameters
#' @param finalF data.frame results of corMirnaRnaMiranda or corMirnaRna function
#' @param ... arguments passed onto pheatmap
#' @param upper_bound is the upper_bound of the correlation pheatmap scale
#'  default is zero user can set to values based on output of correlation result (value)
#' @param main is the title of the pheatmap
#' @param color default inferno(50) from the library viridis R base,
#'  R colorbrewer and viridis compatible
#' @param fontsize default is 7 user adjustable
#' @return pheatmap Obj
#' @export
#' @keywords heatmap, pheatmap, color, correlation plot,correlation_plot
#' @examples
#' x <- mirRnaHeatmap(corr_0)
mirRnaHeatmap <- function(finalF, ..., upper_bound = 0,
                          main = "Default mRNA miRNA heatmap",
                          color = c(viridis::inferno(50), "grey90"), fontsize = 7) {
    dfinalF <- dcast(finalF, V1 ~ V2, fun.aggregate = mean)
    dfinalF[is.na(dfinalF)] <- upper_bound
    rownames(dfinalF) <- dfinalF$V1
    dfinalF <- dfinalF %>% dplyr::select(-V1)
    p <- pheatmap::pheatmap(dfinalF,
        color = color, fontsize = fontsize,
        main = main, ...
    )
    return(p)
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#' @importFrom grDevices colorRampPalette
#' @importFrom corrplot corrplot
NULL

#' drawCorPlot correlation plots for mRNA and miRNA regression results
#'
#' This function plots correlations for mRNA and miRNAs regression results (negative correlation for multi and
#'  individual interactions and positive and negative for interactions)
#' @param corMatrix Significant correlation matrix
#' @param ...  parameters form the corrplot package
#' @return miRNA mRNA target correlation plot
#' @export
#' @keywords R correlation plot
#' @examples
#' \donttest{
#' x <- drawCorPlot(corMatrix)
#' }
#'
drawCorPlot <- function(corMatrix, ...) {
    col2 <- colorRampPalette(
        c(
            colorRampPalette(c(
                "black",
                "#543005",
                "#8c510a",
                "#f46d43",
                "#762a83",
                "#9970ab",
                "#c2a5cf",
                "#e7d4e8",
                "#f7f7f7",
                "#d9f0d3",
                "#a6dba0",
                "#5aae61",
                "#1b7837",
                "#a6d96a",
                "#9ecae1",
                "#3182bd",
                "#08306b"
            ))(50),
            "grey50",
            colorRampPalette(c("red", "#fde0dd", "#fcc5c0", "#fa9fb5", "pink"))(50)
        )
    )
    corrplot(corMatrix, ..., col = col2(211))
}

#' This is data is the mRNA expression across samples and miRNA expression data which is to be
#' investigated in one file. This data set is used in documentation examples.
#'
#' @name Combine
#' @docType data
#' @keywords data
NULL

#' This is data is the mRNA FC and miRNA FC correlation data.
#' This data set is used in documentation examples.
#'
#' @name corr_0
#' @docType data
#' @keywords data
NULL

#' This is data is the mRNA FC and miRNA FC correlation/interaction data results after filteration.
#' This data set is used in documentation examples.
#'
#' @name final_results
#' @docType data
#' @keywords data
NULL

#' This is data is the mRNA expression across samples and miRNA expression data which is to be
#' investigated giving directions on which data is miRNA and which is mRNA.
#' This data set is used in documentation examples.
#'
#' @name geneVariant
#' @docType data
#' @keywords data
NULL

#' This is data is the mRNA FC and miRNA FC correlation/interaction original data.
#' This data set is used in documentation examples.
#'
#' @name inter0
#' @docType data
#' @keywords data
NULL

#' This is data is the mRNA expression file.
#' This data set is used in documentation examples.
#'
#' @name mRNA
#' @docType data
#' @keywords data
NULL

#' This is data is the miRNA expression file.
#' This data set is used in documentation examples.
#'
#' @name miRNA
#' @docType data
#' @keywords data
NULL

#' This is data is the mRNA0_2 FC for 0-2 time point.
#' This data set is used in documentation examples.
#'
#' @name mRNA0_2
#' @docType data
#' @keywords data
NULL

#' This is data is the mRNA2_5 FC for 2-5 time point.
#' This data set is used in documentation examples.
#'
#' @name mRNA2_5
#' @docType data
#' @keywords data
NULL

#' This is data is the mRNA0_5 FC for 0-5 time point.
#' This data set is used in documentation examples.
#'
#' @name mRNA0_5
#' @docType data
#' @keywords data
NULL

#' This is data is the combined mRNA FC for all time points.
#' This data set is used in documentation examples.
#'
#' @name mRNA_fc
#' @docType data
#' @keywords data
NULL

#' This is data is the combined miRNA FC for all time points.
#' This data set is used in documentation examples.
#'
#' @name miRNA_fc
#' @docType data
#' @keywords data
NULL

#' This is data is the miRNA0_2 FC for 0-2 time point.
#' This data set is used in documentation examples.
#'
#' @name miRNA0_2
#' @docType data
#' @keywords data
NULL

#' This is data is the miRNA2_5 FC for 2-5 time point.
#' This data set is used in documentation examples.
#'
#' @name miRNA2_5
#' @docType data
#' @keywords data
NULL

#' This is data is the miRNA0_5 FC for 0-5 time point.
#' This data set is used in documentation examples.
#'
#' @name miRNA0_5
#' @docType data
#' @keywords data
NULL

#' This is data is the output file resulted from time point/conditions background correlation model.
#' This data set is used in documentation examples.
#'
#' @name outs
#' @docType data
#' @keywords data
NULL

#' This is data is the output file resulted from time point/conditions background difference/interelation model.
#' This data set is used in documentation examples.
#'
#' @name outs2
#' @docType data
#' @keywords data
NULL

#' This is data is the output file resulted from time point or conditions or correlation or interrelation model.
#' This data set is used in documentation examples.
#'
#' @name results
#' @docType data
#' @keywords data
NULL

#' This is data is the output file resulted from time point or conditions experiment for interrelation model after filtering and threshold modification.
#' This data set is used in documentation examples.
#'
#' @name sig_InterR
#' @docType data
#' @keywords data
NULL

#' This is data is the output file resulted from time point or conditions experiment for correlation model after filtering and threshold modification.
#' This data set is used in documentation examples.
#'
#' @name sig_corrs
#' @docType data
#' @keywords data
NULL

#' This is data is the results file from regression analysis and its estimates.
#' This data set is used in documentation examples.
#'
#' @name some_model
#' @docType data
#' @keywords data
NULL

#' This is data is the results file from mouse miRanda getInputSpecies function.
#' This data set is used in documentation examples.
#'
#' @name miRandaM
#' @docType data
#' @keywords data
NULL

#' This is data is the results file from EBV miRanda getInputSpecies function.
#' This data set is used in documentation examples.
#'
#' @name miRanda
#' @docType data
#' @keywords data
NULL

#' This data is the mRNA fold change data set for difference or interrelation section.
#' This data set is used in documentation examples.
#'
#' @name mRNA_fc2
#' @docType data
#' @keywords data
NULL

#' This data is the miRNA fold change data set for difference or interrelation section.
#' This data set is used in documentation examples.
#'
#' @name miRNA_fc2
#' @docType data
#' @keywords data
NULL

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#' modelsFilter Filter a list of models based on logical expression
#'
#' This function can be used to filter a list of models (such as returned by \code{runModelsZInf()})
#' based on a logical expression.
#' @param models list of models and related elemenets, such as returned by \code{runModelsZInf()}
#' @param expr expresion that yields a logical vector (evaluated in the environmnet of \code{model})
#' @param quiet suppress warnings
#' @return \code{models} but with all elements filtered by logical expression \code{expr}. Elements
#'         for which filter could not be applied (e.g. length mismatch between element and condition)
#'         are set to \code{NA}.
#' @export
#' @keywords R models filter
#' @examples
#' \donttest{
#' x <- modelsFilter(models, pvalues < 0.05)
#' x <- modelsFilter(models, is_significant)
#' x <- modelsFilter(models, is_significant == FALSE)
#' }
modelsFilter <- function(models, expr, quiet = FALSE) {
    assertthat::assert_that(is.list(models)) # `models` must be a list
    cond <- eval(substitute(expr), envir = models, enclos = parent.frame(n = 1))
    if (!is.logical(cond)) {
        stop(paste0("expression '", deparse(substitute(expr)), "' does not yield logical value"))
    }
    for (i in seq_len(length(models))) {
        if (length(cond) != length(models[[i]])) {
            if (identical(quiet, FALSE)) {
                name <- names(models)[i]
                if ("" != name) {
                    warning(sprintf("condition length mismatch ('%s')", name))
                } else {
                    warning(sprintf("condition length mismatch (i=%d)", i))
                }
            }
            models[[i]] <- NA
        } else {
            models[[i]] <- models[[i]][which(cond)]
        }
    }
    return(models)
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#' @importFrom utils read.table download.file
#' @importFrom data.table fread
#' @import R.utils
NULL

#' importMirandaFile Read internal Miranda file
#'
#' Reads internal Miranda file from extdata and returns it as a data.frame
#' @param fn filename
#' @return data.frame containing Miranda data
#' @examples
#' \donttest{
#' x <- importMirandaFile("Mouse_miRanda.txt")
#' }
importMirandaFile <- function(fn) {
    fn <- system.file("extdata", "miRandaPrepFiles", fn, package = "mirTarRnaSeq", mustWork = TRUE)
    ret1 <- read.table(fn, as.is = TRUE, sep = "\t")
    return(ret1)
}


#' downloadMirandaFile Read internal Miranda file
#'
#' Reads internal Miranda file from extdata and returns it as a data.frame
#' @param urlf URL of the specific chosen file
#' @return data.frame containing downloaded miRanda file
#' @examples
#' \donttest{
#' x <- downloadMirandaFile(
#'        "https://zenodo.org/record/4615670/files/Mouse_miRanda.txt.gz"
#'      )
#' }
downloadMirandaFile <- function(urlf) {
    return(downloadMirandaFile_(urlf))
}

# use internal function here so we can R.cache it without changing how it looks like for the user.
downloadMirandaFile_ <- function(urlf) {
    # make effective statement error if it returns true or false for http with or without s
    assert_that(grepl("^https?://.*$", urlf))
    tmp <- tempfile(fileext = ".gz")
    op <- options(timeout = 99999)
    on.exit({
        unlink(tmp)
        options(op)
    })
    download.file(urlf, tmp)
    ret1 <- fread(tmp, sep = "\t", header = FALSE)
    return(as.data.frame(ret1))
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#' @importFrom stats cor
NULL

## quiet concerns of R CMD check regarding unbound global variables (in dplyr::filter() calls)
if (getRversion() >= "2.15.1") {
    utils::globalVariables(c("V1", "V2"))
}

#' corMirnaRna correlation for miRNA and mRNA
#'
#' This function uses the output of one2OneRnaMiRNA and returns the correlation dataframe
#' @param mRNA mRNA file generated from foldchanges (FC) obj of the one2OneRnaMiRNA
#' @param miRNA miRNA file generated from foldchanges (FC) obj of the one2OneRnaMiRNA
#' @param method Default is "pearson" else use "kendall" or "spearman"
#' @return Correlation data.frame
#' @export
#' @keywords Correlation with miRanda, miRanda Threshold
#' @examples
#' x <- corMirnaRna(mRNA_fc, miRNA_fc, method = "spearman")
corMirnaRna <- function(mRNA, miRNA, method = "pearson") {
    tmRNA <- t(mRNA)
    tmiRNA <- t(miRNA)
    mycor <- cor(cbind(tmRNA, tmiRNA), method = method)
    mmycor <- reshape2::melt(mycor)
    names(mmycor) <- c("V1", "V2", "value")
    # Makes a square matrix the correlation should not be squared
    mycorf <- dplyr::filter(mmycor, !(V2 %in% rownames(miRNA)))
    mycorf <- dplyr::filter(mycorf, !(V1 %in% rownames(mRNA)))
    mycorf$V1 <- as.character(mycorf$V1)
    mycorf$V2 <- as.character(mycorf$V2)
    return(as.data.frame(mycorf))
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#'  combiner combines the miRNA and mRNA files
#'
#' This function makes and intersection dataframe for mRNA and miRNA/s of interest to be tested.
#' @param mRNA Matrix or data.frame mRNA/RNA from transforemed diff expression file (genrated
#' using TZtranz)
#' @param miRNA Matrix or data frame miRNA from transforemed diff file (genrated
#' using TZtranz)
#' @param miRNA_select A vector of character's for miRNAs which the user is interested in investigating if glm is use 1 miRNA
#' should be input. If multivariate several miRNAs should be imported, same goes for interaction determination for miRNAs. Note
#' we do not recommend more than 3-4 miRNAs at a time for the latter cases.
#' @return A dataframe which includes only mRNAs and miRNA intersection for the next estimation geneVari output.
#' @export
#' @keywords miRNA_mRNA_Intersection
#' @examples
#' miRNA_select <- c("ebv-mir-bart9-5p")
#' x <- combiner(mRNA, miRNA, miRNA_select)
combiner <- function(mRNA, miRNA, miRNA_select) {
    sub_miRNA <- miRNA[miRNA_select, ]
    common_samples <- intersect(names(mRNA), names(sub_miRNA))
    combination <- as.data.frame(t(rbind(mRNA[, common_samples], sub_miRNA[, common_samples])))
    combination <- combination[, !apply(is.na(combination), 2, any)]
    return(combination)
}

#' ## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020
#'
#' #' @importFrom rsq rsq
#' NULL
#'
#' #
#' #' R-squared from model
#' #'
#' #' Obtrain R-squared from a linear model
#' #' @param model linear model
#' #' @export
#' #' @keywords linear model, r-squared
#' #' @examples
#' #' \donttest{
#' #' x <- modelRsquared(model)
#' #' }
#' modelRsquared <- function(model) {
#'   return(rsq(model))
#'  }

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#' @importFrom stats var formula
NULL

#' runModels runs miRNA mrna model model for various miRNA-mRNA data distributions
#'
#' This function defines the boundaries of mRNA vs miRNAs of interest to be analysed by the runModels function
#' @param combination the combined file for mRNA and selected miRNAs output of combiner function
#' @param select_mRNA the output of gene_variant function.
#' @param select_miRNA The vector of miRNA/s to be investigated.
#' @param mode the mode of analysis if more than one miRNA is being investigated multivariate "multi"
#' or co-variate/interaction analysis "inter" is being used
#' @param family gaussian or poisson
#' @param scale factor to scale input data (for genes) by, prior to rounding and model
#'              fitting. (\code{scale} must be greater than zero).
#' @param cutoff p-value cut off to call significance
#' @param all_coeff if true only models with all negative coefficients will be selected if false at least one
#' @return A list of p-vlaues, annova, and significance for each gene and the miRNA/s of interest
#' @export
#' @keywords runModels univariate multivariate interaction glm
#' @examples
#' x <- runModels(Combine, geneVariant, "ebv-mir-bart9-5p")
runModels <- function(combination, select_mRNA, select_miRNA, mode = NULL,
                      family = glm_poisson(), scale = 1, cutoff = 0.05,
                      all_coeff = NULL) {
    assertthat::assert_that(is.numeric(scale) && 1 == length(scale) && scale > 0)
    assertthat::assert_that(is.numeric(cutoff) && 1 == length(cutoff) && cutoff > 0)
    assertthat::assert_that(is.null(all_coeff) || (is.logical(all_coeff) && 1 == length(all_coeff)))
    is_significant <- c()
    all_models <- list()
    genes <- c()
    pvalues <- c()
    AICvalues <- c()
    # remove zero-variance columns.
    combination <- combination[, apply(combination, 2, var) != 0, drop = FALSE]
    select_mRNA <- intersect(select_mRNA, colnames(combination))

    for (gene in select_mRNA) {
        model_formula <- formula(paste(sprintf("`%s`", gene),
            makeFormulaRightSide(select_miRNA, mode = mode),
            sep = " ~ "
        ))

        # run the model for a mRNA
        x <- runModel(model_formula, data = combination, model = family, scale = scale)

        # check for negative coeffs (if 'all_coeff' is used)
        keep <- TRUE
        if (!is.null(all_coeff)) {
            if (identical(all_coeff, TRUE)) {
                if (!(all(modelCoefficients(x) < 0))) {
                    keep <- FALSE
                }
            } else {
                if (!(any(modelCoefficients(x) < 0))) {
                    keep <- FALSE
                }
            }
        }

        # if we were unable to make a model (i.e. it is NULL) or we didn't get
        # the negative coeffs we wanted, we don't add it to the lists here.
        if (!is.null(x) && keep) {
            ret <- modelTermPvalues(x)
            is_significant <- c(is_significant, any(is.finite(ret) & ret < cutoff))
            pvalues <- c(pvalues, list(ret))
            all_models <- c(all_models, list(x))
            genes <- c(genes, gene)
            AICvalues <- c(AICvalues, list(modelAIC(x)))
        }
    }
    names(is_significant) <- genes
    names(pvalues) <- genes
    names(all_models) <- genes
    names(AICvalues) <- genes

    # turn p-values into data.frame (with columns for every mirna)
    pvalues <- t(as.data.frame(pvalues))
    colnames(pvalues) <- gsub("^`?(.*)[^`]`?$", "\\1", colnames(pvalues)) # remove ` ` from colnames...
    rownames(pvalues) <- genes
    return(
        list(
            is_significant = is_significant, # logical vector TRUE if any term sign.; FALSE otherwise.
            genes = genes,
            pvalues = pvalues,
            all_models = all_models,
            AICvalues = AICvalues,
            select_mRNA = select_mRNA,
            select_miRNA = select_miRNA,
            combination = combination
        )
    )
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#' @importFrom stats cor
NULL

## quiet concerns of R CMD check regarding unbound global variables (in dplyr::filter() calls)
if (getRversion() >= "2.15.1") {
    utils::globalVariables(c("V1", "V2", "value"))
}

#' corMirnaRnaMiranda correlation for miRNA and mRNA
#'
#' This function uses the output of one2OneRnaMiRNA and returns the correlation dataframe.
#' @param mRNA mRNA file generated from foldchanges (FC) obj of the one2OneRnaMiRNA.
#' @param miRNA miRNA file generated from foldchanges (FC) obj of the one2OneRnaMiRNA
#' @param CorVal Correlation cut off.Example: If correlation -0.2 it would only return correlations with
#' smaller than this value correlation for miRNA and mRNA at various time points.
#' @param getInputSpeciesDF The dataframe generated from the getInputSpecies function.
#' @param method Default is "pearson" else use "kendall" or "spearman".
#' @return Correlation dataframe
#' @export
#' @keywords Correlation with miRanda, miRanda Threshold
#' @examples
#' x <- corMirnaRnaMiranda(mRNA_fc, miRNA_fc, Cor = -0.9, miRandaM)
corMirnaRnaMiranda <- function(mRNA, miRNA, CorVal, getInputSpeciesDF, method = "pearson") {
    tmRNA <- t(mRNA)
    tmiRNA <- t(miRNA)
    mycor <- cor(cbind(tmRNA, tmiRNA), method = method)
    mmycor <- reshape2::melt(mycor)
    names(mmycor) <- c("V1", "V2", "value")
    mmycor$V1 <- as.character(mmycor$V1)
    mmycor$V2 <- as.character(mmycor$V2)
    mycorf <- dplyr::filter(mmycor, !(V2 %in% rownames(miRNA)))
    mycorf <- dplyr::filter(mycorf, !(V1 %in% rownames(mRNA)))
    fmycorf <- dplyr::filter(mycorf, value < CorVal)
    Lfmycorf <- dplyr::left_join(getInputSpeciesDF, fmycorf, by = c("V1", "V2"))
    FLfmycorf <- Lfmycorf[is.finite(Lfmycorf$value), ]
    return(as.data.frame(FLfmycorf))
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#' Sparse Partial Correlations On mRNA/miRNA Expression
#' We make mirTarRnaSeq compatible to SPONGE package in order to estimate
#' sparse matrix correlation (using elstic net) for prediction potential miRNA-mRNA
#' interaction. Note this function/method is suggested for miRNA/mRNA interactions
#' in many samples with a notable variance of mRNA/miRNA expression. This model also
#' only reports negative sparse partial correlation predictions.
#' @param mirna_exp miRna expression data.frame with miRNA for rows and samples for columns
#' @param diff_exp mRNA expression data.frame with mRNA for rows and samples for columns
#' @param miranda_sponge_predict miRanda sponge compatible matrix produced by miranda_sponge_predict function
#' @param non_null  The default for this parameter is TRUE, hence it returns only non-null estimated if
#' FALSE it would return all NULL and TRUE estimates.
#' @return matrix adjacency matrix with column names miRNA and row names mRNA
#' @keywords Sponge, mirTarRnaSeq, sparse partial correlation, ceRNA
#' @export
one2manySponge <- function(mirna_exp, diff_exp, miranda_sponge_predict,non_null=TRUE){
  # Attempt to load SPONGE
  if (!requireNamespace("SPONGE", quietly = TRUE)) {
    stop("SPONGE package unavailable")
  }
  mirna_exp_ad<-t(mirna_exp)
  mrna_exp_ad<-t(diff_exp)
  rowing_mRNA<-rownames(mrna_exp_ad)
  adequate_miRNA<- mirna_exp_ad[rownames(mirna_exp_ad) %in% rowing_mRNA, ]
  new_meth_mRNA <- mrna_exp_ad[ order(rownames(mrna_exp_ad)), ]
  new_meth_miRNA<-adequate_miRNA[ order(rownames(adequate_miRNA)), ]
  sponging<-SPONGE::sponge_gene_miRNA_interaction_filter(
    gene_expr = new_meth_mRNA,
    mir_expr = new_meth_miRNA,
    mir_predicted_targets = miranda_sponge_predict
  )
  if (non_null) {
    sponging <- sponging[!sapply(sponging, is.null)]
  }
  return(sponging)
}

## Written by Mercedeh Movassagh <mercedeh@ds.dfci.harvard.edu>, Aug 2020

#' @importFrom stats quantile
NULL

#' threshSig Using shuffling threshold finds appropriate significant miRNA-mRNA correlation
#'
#' This function uses the sampCorRnaMirna shuffled output to determine an appropriate threshold
#' for significant mRNA and miRNA relationship of the dataset and shows all those with significant
#' relationships.
#' @param corr0 data.frame results of corMirnaRna function.
#' @param corrS vector of correlations, from the sampCorRnaMirna function.
#' @param pvalue The p value threshold to be used on the sampled data.
#' @return A dataframe of Significant mRNA and miRNA
#' @export
#' @keywords Significance, Threshold
#' @examples
#' \donttest{
#' x <- mirRnaHeatmap(outs, corr_0)
#' }
#'
threshSig <- function(corr0, corrS, pvalue = 0.05) {
    threshold <- quantile(corrS, pvalue) # 5% default
    sig_corrs <- corr0[corr0$value < threshold, ]
    return(sig_corrs)
}

---
title: "mirTarRnaSeq"
author: "Mercedeh Movassagh"
package: mirTarRnaSeq
output:
  BiocStyle::pdf_document
vignette: >
  %\VignetteIndexEntry{mirTarRnaSeq}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
---
```{r style, echo=FALSE, results='asis'}
BiocStyle::markdown()
```

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  dpi=300,
  warning = FALSE,
  collapse = TRUE,
  error = FALSE,
  comment = "#>",
  echo=TRUE
)
library(mirTarRnaSeq)
library(knitr)
library(rmarkdown)
library(mirTarRnaSeq)
library(dplyr)
library(ggplot2)
library(reshape2)
library(tidyr)
library(viridis)

doctype <- opts_knit$get("rmarkdown.pandoc.to")
```

# Introduction

mirTarRnaSeq is a package for miRNA and mRNA interaction analysis through regression and correlation approaches supporting various modeling approaches (gaussian, poisson, negative binomial, zero inflated poisson or negative binomial models for the data). miRNA and mRNA sequencing data are analysed from the same experiment (condition or time point data) and mRNA targets of the miRNAs from same experiment will be identified using statistical approaches.

The example data set for the first approach is 25 matching miRNA and mRNA EBV positive samples from TCGA identified as high EBV load based on Movassagh et al, Scientific Reports, 2019 paper. We attempt to identify the EBV miRNA targets on EBV genome (part1).

The second example set set is the simulated mouse fibroblast differentiated to muscle cells in three time points. Here, we try to identify mRNA targets of miRNA expressed at various time points (parts 2 and 3).

## Data upload

### mirTarRnaSeq accepts data in dataframe or table formats

* For the first approach ([Part1](#part1-uploading-data-into-the-application)) we use a table of differential expressed mRNA genes in EBV from TCGA stomach cancer samples with high levels of EBV miRNA expression.
* Next we use a list of normalized (tpm) EBV miRNA expression data from the same samples.The user has the option to use count data and model accordingly or use tzTrans() function for zscore normalization and then model.
* For the second part (Part2, Part3) of the experiment we are using two tables of differentially expressed mRNA and miRNA sequencing fold change results from mouse time point specific differentiation experiments.
* The example data are also available at https://doi.org/10.5281/zenodo.6371713.

# Part1 - miRNA mRNA regressions across sample cohorts
The users can utilize TPM/RPKM or count data for this section of analysis as long as if the data is normalized (TPM/RPKM) is used for the mRNA expression files the miRNA expression files are also normalized (TPM). The format of the files for miRNA and mRNA needs to be odd the class dataframe with the sample names as colnames and mRNA names as rownames. Note, for this section of analysis we have only included all differentially expressed mRNAs for the analysis but the user can included all mRNAs. However, the user should realize if they choose to use all mRNAs expressed the analysis will take a longer time.

## Uploading data into the application. The example data can be found in the test folder under package.

```{r eval=TRUE, echo=TRUE}
# Helper function to access test files in extdata
get_test_file <- function(file_name) {
  if (missing(file_name)) {
    return(system.file("extdata", "test", package="mirTarRnaSeq"))
  }
  return(system.file("extdata", "test", file_name, package="mirTarRnaSeq"))
}

# Read input files
DiffExp<-read.table(get_test_file("EBV_mRNA.txt.gz"), as.is=TRUE, header=TRUE, row.names=1)
miRNAExp<-read.table(get_test_file("EBV_miRNA.txt.gz"), as.is=TRUE, header=TRUE, row.names=1)
```

## Get miRanda file

We currently support miRanda runs (potential miRNA target parsing by score,interaction energy, and interaction or miRNA length ) on seven species ("Human", "Mouse", "Drosophila", "C.elegans", "Epstein_Barr" (EBV), "Cytomegalovirus" (CMV) and "Kaposi_Sarcoma" (KSHV)). We also support the viral miRNAs targeting human genes for EBV "Epstein_Barr_Human", CMV "CMV_Human" and KSHV "KSHV_Human".
1) Here we first import the relevant miRanda file .
2) We only keep targets which are also targets of EBV miRNAs based on our EBV miRanda file.
Note, for other organisms (not provided by the pacakged) the appropriate format for miRanda input file is V1 column: name of the miRNA, V2 column: name of the predicted gene, V3: column score/rank of the miranda interaction (or any other score for interaction if TargetScan ( for TargetScan prediction user can provide context++ score and threshold accordingly) is used, note these three columns are required), V4 column: folding energy of miRNA-mRNA interaction. V5 column: target Identity value and V6 column: miRNA idnetity value. (Note, all these values are available after miRanda analysis but V1-V3 must be provided not matter the prediction method)

```{r eval=TRUE, echo=TRUE}
miRanda <- getInputSpecies("Epstein_Barr", threshold = 140)
DiffExpmRNASub <- miRanComp(DiffExp, miRanda)
```

## Select miRNA

```{r eval=TRUE, echo=TRUE}
miRNA_select<-c("ebv-mir-BART9-5p")
```

## Combine the mRNA and miRNA file and define boundaries ans specify which mRNA and miRNA files in the combined file.

```{r eval=TRUE, echo=TRUE}
Combine <- combiner(DiffExp, miRNAExp, miRNA_select)
geneVariant <- geneVari(Combine, miRNA_select)
```

Running various univariate models(1 to 1 miRNA-mRNA relationships) with various miRNA-mRNA distribution assumptions for either 1 miRNA and 1 mRNA relationship chosen/selected by the user or across all miRNAs and mRNAs in the expression dataframes. Note,the users can choose to run any of the available distribution model assumptions (glm_poisson, glm_gaussian, glm_nb (negative binomial), glm_zeroinfl (zero inflated model or zero inflated negative binomial)).

## Run a one to one miRNA/mRNA gaussian regression model (univariate model for 1 miRNA and 1 mRNA).
Here LMP_1 is EBV mRNA and the ebv-mir-BART9-5p is the miRNA and we are running a glm poisson model.

```{r eval=TRUE, echo=TRUE}
j <- runModel(`LMP-1` ~ `ebv-mir-BART9-5p`,
              Combine, model = glm_poisson(),
              scale = 100)
# Print P value of poisson model of association between between LMP_1 and
# ebv-mir-BART9-5p
print(modelTermPvalues(j))
```

## Running Gaussian model over all individual miRNA mRNA models (univatiate model for every mRNA and miRNA relationship across the input dataset with Gaussian distribution assumptions)

```{r gauss, eval=TRUE, echo=TRUE,out.width="90%",out.height="90%", fig.align="center"}
blaGaus <- runModels(Combine,
                     geneVariant, miRNA_select,
                     family = glm_gaussian(),
                     scale = 100)
# Plot the regression relationship for the mRNA and miRNAs (BHLF1 is the EBV
# mRNA). Note, these are standard quality check plot outputs from plot
# regression.
par(oma=c(2,2,2,2))
par(mfrow=c(2,3),mar=c(4,3,3,2))
plot(blaGaus[["all_models"]][["BHLF1"]])
plot(modelData(blaGaus[["all_models"]][["BHLF1"]]))
# To comprehend the performance of all models we look at Akaike information
# criterion (AIC) across all miRNA-mRNA model performances and then look at
# the density plots. Note, the models with lower comparitive AICs have better
# performace.
G <- do.call(rbind.data.frame, blaGaus[["AICvalues"]])
names(G) <- c("AIC_G")
# Low values seems like a reasonable model
plot(density(G$AIC_G))
# Print out the AIC of all miRNA-mRNA models ( All observed AIC values for the
# miRNA-mRNA models)
GM <- melt(G)
```

## Running poisson model over all individual miRNA mRNA models (univatiate model for every mRNA and miRNA relationship across the input dataset with poisson distribution assumptions)

```{r poisson, eval=TRUE, echo=TRUE, out.width="90%",out.height="90%", fig.align="center"}
blaPois <- runModels(Combine,
                     geneVariant, miRNA_select,
                     family = glm_poisson(),
                     scale = 100)
par(oma=c(2,2,2,2))
par(mfrow=c(2,3),mar=c(4,3,3,2))
plot(blaPois[["all_models"]][["LMP-2A"]])
plot(modelData(blaPois[["all_models"]][["LMP-2A"]]))
P <- do.call(rbind.data.frame, blaPois[["AICvalues"]])
names(P) <- c("AIC_Po")
PM <- melt(P)
```

## Running negative binomial model over all individual miRNA mRNA models (univatiate model for every mRNA and miRNA relationship across the input dataset with negative bionomial distribution assumptions).

```{r negbin, eval=TRUE, echo=TRUE, out.width="70%",out.height="90%", fig.align="center"}
blaNB <- runModels(Combine,
                   geneVariant, miRNA_select,
                   family = glm_nb(), scale = 100)
par(mar=c(4,3,3,2))
plot(modelData(blaNB[["all_models"]][["BALF1"]]))
B <- do.call(rbind.data.frame, blaNB[["AICvalues"]])
names(B) <- c("AIC_NB")
BM <- melt(B)
```

## Running zero inflated negative binomial model over all individual miRNA mRNA models (univatiate model for every mRNA and miRNA relationship across the input dataset with zero inflated negative bionomial distribution assumptions).

```{r zeroinfnegbin, eval=TRUE, echo=TRUE, out.width="70%",out.height="90%", fig.align="center"}
blazeroinflNB <- runModels(Combine, geneVariant,
                            miRNA_select,
                            family = glm_zeroinfl(dist = "negbin"),
                            scale = 100)
# To test AIC model performance
ZNB <- do.call(rbind.data.frame, blazeroinflNB[["AICvalues"]])
names(ZNB) <- c("AIC_ZNB")
par(mar=c(4,3,3,2))
plot(density(ZNB$AIC_ZNB))
ZNBM<-melt(ZNB)
```

## Running zero inflated poisson binomial model over all individual miRNA mRNA models (univatiate model for every mRNA and miRNA relationship across the input dataset with zero inflated poisson distribution assumptions).

```{r zeroinfpoisson, message=FALSE, echo=TRUE, cache=FALSE, results=TRUE, out.width="70%",out.height="90%", fig.align="center"}
blazeroinfl <- runModels(Combine, geneVariant,
                         miRNA_select,
                         family = glm_zeroinfl(),
                         scale = 100)
# To test AIC model performance
Zp <- do.call(rbind.data.frame, blazeroinfl[["AICvalues"]])
names(Zp) <- c("AIC_Zp")
par(mar=c(4,3,3,2))
plot(density(Zp$AIC_Zp))
ZpM <- melt(Zp)
```

## Including Plots for all models to decide which to use

```{r plots, eval=TRUE, echo=TRUE, fig.width = 5, fig.height=5, out.width="80%", dpi=300, fig.align="center"}
bindM <- rbind(PM, BM, GM, ZpM, ZNBM)
p2 <- ggplot(data = bindM, aes(x = value, group = variable,
                               fill = variable)) +
  geom_density(adjust = 1.5, alpha = .3) +
  xlim(-400, 2000)+
  ggtitle("Plot of of AIC for ebv-mir-BART9-5p regressed all mRNAs ")+
  ylab("Density")+ xlab ("AIC Value")
p2
```

## The user can decide to use runModels() with glm_multi() (with multi and inter mode options)
When using glm_multi(), where all available models will be run, the AICs will be compared and the best model will be chosen based on the miRNA-mRNA model AIC score. In the example bellow we are using the mode= "multi" option for combination of 2 miRNAs (multivariate model) for interaction model the user can choose the mode= "inter" option. Note all_coeff parameters defaults TRUE all interactions (only negative miRNA-mRNA relationships) are reported. More comments on the mode is provided in the next section of the vignette. If all miRNA-mRNA relationships are wanted this parameter can be set to false.

```{r message=FALSE,echo=TRUE, cache=FALSE, results=TRUE, warning=FALSE, comment=TRUE, warning=FALSE}
miRNA_select<-c("ebv-mir-BART9-5p","ebv-mir-BART6-3p")
Combine <- combiner(DiffExp, miRNAExp, miRNA_select)
geneVariant <- geneVari(Combine, miRNA_select)
MultiModel <- runModels(Combine, geneVariant,
                        miRNA_select, family = glm_multi(),
                        mode="multi", scale = 10)
# Print the name of the models used for the analysis (note the printed outputs
# are the number of models ran by various models based on the AIC scores using
# the glm_multi())
print(table(unlist(lapply(MultiModel$all_models, modelModelName))))
```

## GLM multi and GLM inter
In GLM multi (multinomial), (model="multi"), the user can choose to run as many selected miRNAs they choose against all mRNAs datasets.In this example we select two miRNAs. The user can also select particular mRNAs to run this analysis on. We recommend the user always chooses the number of miRNAs for the multinational models and not run it across all the dataset as the analysis could take a very long time. If the user chooses to use more than two miRNAs for the multinomial model they should assign specific miRNAs and we recommend running it on a high memory machine. In GLM multi (synergy) model (mode="inter"), miRNA interactions. In this example we select two miRNAs. The user can also select particular mRNAs to run this analysis on. We recommend the user always chooses the number of miRNAs for the multinational models and not run it across all the dataset as the analysis could take a very long time. If the user chooses to use more than two miRNAs for the multinomial model they should assign specific miRNAs and we recommend running it on a high memory machine.

```{r message=FALSE,echo=TRUE, cache=FALSE, results=TRUE, warning=FALSE, comment=TRUE, warning=FALSE}
miRNA_select<-c("ebv-mir-BART9-5p","ebv-mir-BART6-3p")
Combine <- combiner(DiffExp, miRNAExp, miRNA_select)
geneVariant <- geneVari(Combine, miRNA_select)
InterModel <- runModels(Combine,
                        geneVariant, miRNA_select,
                        family = glm_multi(
                          models=list(glm_gaussian,
                          glm_poisson())),mode="inter", scale = 10)

# Print the name of the models used for the analysis (note, you can see although
# we have defined for the models to be run using either gaussian or poisson
# options, mirTarRnaSeq chooses the poisson model for all as it performs with a
# better AIC (lower AIC value) for all miRNA-mRNA interactions)
print(table(unlist(lapply(InterModel$all_models, modelModelName))))
```

## Running all miRNA  and mRNA combinations at the same time
Note, for "inter" and "multi" mode options we only support combination of 2 if more than two relationships are of interest, we recommend selecting the miRNAs and running the previously described runModels function due to complication of the models and time consumption.

```{r message=FALSE,echo=TRUE, cache=FALSE, results=TRUE, warning=FALSE, comment=TRUE}
vMiRNA<-rownames(miRNAExp)
# Note, the user can run all miRNAs but for speed reasons we have chosen the
# first 5 here for mirnas input for the analysis.
All_miRNAs_run<-runAllMirnaModels(mirnas =vMiRNA[1:5] ,
                                  DiffExpmRNA = DiffExpmRNASub,
                                  DiffExpmiRNA = miRNAExp,
                                  miranda_data = miRanda,prob=0.75,
                                  cutoff=0.05,fdr_cutoff = 0.1, method = "fdr",
                                  family = glm_multi(), scale = 2, mode="multi")

#select significant genes
hasgenes <- lapply(All_miRNAs_run, function(x) nrow(x$SigFDRGenes)) > 0
All_miRNAs_run <- All_miRNAs_run[hasgenes]
print(table(unlist(lapply
                   (All_miRNAs_run[[1]][["FDRModel"]][["all_models"]],
                     modelModelName))))
# Print specific models for specific miRNAs (in this example the significant
# multivariate model for ebv-mir-BART1-5p and ebv-mir-BART11-3p )
print(
  table(
    unlist(lapply(
      (All_miRNAs_run[["ebv-mir-BART1-5p and ebv-mir-BART11-3p"]]
                     [["FDRModel"]]
                     [["all_models"]]),
        modelModelName))))
```

## One2manySponge
miRNA-mRNA sparse partial correlation prediction using elastic net and compatibility with SPONGE package.Note, we do not recommend using this method
for low number of samples (20 or less), or in miRNAs/mRNAs with low variance as the sample size increases the confidence in this analysis increases same goes for miRNA and mRNA variance. Optimal analysis using this method is in 100 or more samples in high variance miRNAs/mRNAs. For viral analysis or samples with low mRNA miRNA variance we recommend using part1 of mirTarRnaSeq or narrowing down the miRNAs and mRNAs to high variance ones.

```{r eval=TRUE, echo=TRUE, results='hide', error=TRUE}
# Make miRanda file compatible with SPONGE package mir_predicted_targets()
sponge_mir_input<-miranda_sponge_predict(miRNAExp,DiffExp,miRanda)

#Perform sparse partial correlation for miRNA-mRNA relationships
one2many_output<-one2manySponge(miRNAExp,DiffExp,sponge_mir_input)
```


# Part2 - Identify miRNA mRNA correlations across 3 or more time points

Note for this analysis we need fold change data for time points or control versus condition. Hence, a differential expression (DE) analysis needs to be performed before proceeding this analysis (These values should be provided for all miRNA and mRNA in the DE expression and not only the significantly DE miRNAs/mRNAs). Here we are looking at differential expression (DE) files between three time points. The format of each timepoint/control vs condition file needs to be Gene/miRNA names as the first column, log2FC or logfoldchange (FC), (or any other FC metrics as long as for both miRNA, and mRNA the same metrics is used) for column two. The pvalue assigned to the gene(mRNA) expression after the differential expression analysis on the third column.For the miRNA file, the user needs to assign Gene names on the first column and the representative log2FC or logfoldchange (FC) on the second column.

Load files from test directory or you can load individually and feed them in separately in a list: list[(mRNA1,mRNA2,mRNA)]

```{r eval=TRUE, echo=TRUE}
files <- local({
  filenames <- list.files(path=get_test_file(), pattern="^.*\\.txt\\.gz$",
                          full.names=TRUE)
  files <- lapply(filenames, read.table, as.is=TRUE, header=TRUE, sep="\t")
  names(files) <- gsub("^.*/(.*)\\.txt\\.gz$", "\\1", filenames)
  return(files)
})
```

## Get mRNAs

```{r eval=TRUE, echo=TRUE}
mrna_files <- files[grep("^mRNA", names(files))]
```

## Get mRNAs with particular fold change

```{r eval=TRUE, echo=TRUE}
mrna_files <- files[grep("^mRNA", names(files))]
mrna <- one2OneRnaMiRNA(mrna_files, pthreshold = 0.05)$foldchanges
```

## Get all miRNAs

```{r eval=TRUE, echo=TRUE}
mirna_files <- files[grep("^miRNA", names(files))]
mirna <- one2OneRnaMiRNA(mirna_files)$foldchanges
```

## Get mRNA miRNA correlation

```{r eval=TRUE, echo=TRUE}
corr_0 <- corMirnaRna(mrna, mirna,method="pearson")
```

## Make a background distribution correlation

```{r eval=TRUE, echo=TRUE, results=FALSE}
outs <- sampCorRnaMirna(mrna, mirna,method="pearson",
                        Shrounds = 100, Srounds = 1000)
```

## Plot density plots
Density plot for background and corrs in our data. Note grey is the background distribution and red is the actual data.
```{r eval=TRUE, echo=TRUE, fig.width = 3, fig.height=3, out.width="80%", dpi=300, fig.align="center"}
#Draw density plot
mirRnaDensityCor(corr_0, outs)
```

## Get correlations below threshold

```{r eval=TRUE, echo=TRUE}
#Identify significant correlation
sig_corrs <- threshSig(corr_0, outs,pvalue = 0.05)
```

## Get mouse miRanda data

```{r eval=TRUE, echo=TRUE}
#Import concordant miRanda file
miRanda <- getInputSpecies("Mouse", threshold = 150)
```

## mRNA miRNA correlation heatmap

Correlation heatmap for cor equal or less than -0.7.
Note upperbound for heatmap should be always less than the correlation threshold.
```{r eval=TRUE, echo=TRUE, fig.width = 8, fig.height=5, out.width="90%", dpi=300, fig.align="center"}
#Extract your target correlations based on miRanda and correlation threshold.
newcorr <- corMirnaRnaMiranda(mrna, mirna, -0.7, miRanda)
mirRnaHeatmap(newcorr,upper_bound = -0.6)
```

## Get intersection of miRanda

Get miRanda intersection and significant miRNA and mRNA interactions and the plot it.
```{r eval=TRUE, echo=TRUE, fig.width = 5, fig.height=4, out.width="80%", dpi=300, fig.align="center"}
#Make final results file for significant
#correlations intersecting with miRanda file
results <- miRandaIntersect(sig_corrs, outs, mrna, mirna, miRanda)
#Draw correlation heatmap
p<- mirRnaHeatmap(results$corr,upper_bound =-0.99)
p
```

# Part3 - Identify significant miRNA mRNA relationships for 2 time points

## Import data
```{r eval=TRUE, echo=TRUE}
files <- local({
  filenames <- list.files(path=get_test_file(), pattern="^.*\\.txt\\.gz$",
                          full.names=TRUE)
  files <- lapply(filenames, read.table, as.is=TRUE, header=TRUE, sep="\t")
  names(files) <- gsub("^.*/(.*)\\.txt\\.gz$", "\\1", filenames)
  return(files)
})
```

## Only look for time point difference 0-5
```{r eval=TRUE, echo=TRUE}
mirna_files <- files[grep("^miRNA0_5", names(files))]
mrna_files <- files[grep("^mRNA0_5", names(files))]
```

## Get fold changes above thereshold
```{r eval=TRUE, echo=TRUE}
# Parse Fold Change Files for P value and Fold Change.
mrna <- one2OneRnaMiRNA(mrna_files, pthreshold = 0.05)$foldchanges
mirna <- one2OneRnaMiRNA(mirna_files)$foldchanges
```

## Estimate miRNA mRNA differences based on Fold Change
```{r eval=TRUE, echo=TRUE}
# Estimate the miRNA mRNA FC differences for your dataset
inter0 <- twoTimePoint(mrna, mirna)
```

## Make  background distribution
```{r eval=TRUE, echo=TRUE, message=FALSE, results=FALSE}
#Make a background distribution for your miRNA mRNA FC differences
outs <- twoTimePointSamp(mrna, mirna,Shrounds = 10 )
```

## miRanda data import
```{r eval=TRUE, echo=TRUE}
#Import concordant miRanda file
miRanda <- getInputSpecies("Mouse", threshold = 140)
```

## Identify relationships below threshold
```{r eval=TRUE, echo=TRUE}
#Identify miRNA mRNA relationships bellow a P value threshold, default is 0.05
sig_InterR <- threshSigInter(inter0, outs)
```

## miRanda intersection with results
```{r eval=TRUE, echo=TRUE}
#Intersect the mirRanda file with your output results
results <- mirandaIntersectInter(sig_InterR, outs, mrna, mirna, miRanda)
```

## Make dataframe and plots
```{r eval=TRUE, echo=TRUE, fig.width = 4, fig.height=4, out.width="90%", dpi=300, fig.align="center"}
#Create a results file for heatmap
final_results <- finInterResult(results)
#Draw plots of miRNA mRNA fold changes for your results file
par(mar=c(4,4,2,1))
drawInterPlots(mrna,mirna,final_results)
```

## mRNA miRNA heatmap of miRNA mRNA FC differences
Heatmap for p value significant miRNA mRNA fold change differences
when compared to backgound
```{r eval=TRUE, echo=TRUE, fig.width = 3, fig.height=2, out.width="80%", dpi=300, fig.align="center"}
CorRes<-results$corrs
#Draw heatmap for miRNA mRNA significant differences
#Note: you do not have to use the upper_bound function unless you want
#investigate a particular range for miRNA mRNA differences/relationships
mirRnaHeatmapDiff(CorRes,upper_bound = 9.9)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/models_run.R
\name{modelModelPvalue}
\alias{modelModelPvalue}
\title{Obtain model p-value}
\usage{
modelModelPvalue(x)
}
\arguments{
\item{x}{fitted model}
}
\value{
Pvalue for the model
}
\description{
Obtain model p-value
}
\examples{
modelModelPvalue(some_model)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/models_zeroinfl.R
\name{glm_zeroinfl}
\alias{glm_zeroinfl}
\title{Model functions for zero inflated model using either Poisson or Negative Binomial distributions.}
\usage{
glm_zeroinfl(dist = "poisson")
}
\arguments{
\item{dist}{either 'poisson' or 'negbin'}
}
\value{
structure containing functions \code{fit}, \code{coefficients},
        \code{aic}, \code{data}, \code{pterm}, \code{pmodel}, and a
        character string "glm_zeroinfl" in \code{model}.
}
\description{
Implements standardaized functions to fit the zero inflated model with
Poisson or Negative Binomial distribution, and to obtain coefficients,
pvalues, etc.
}
\examples{
x <- glm_zeroinfl("negbin")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getInputSpecies.R
\name{getInputSpecies}
\alias{getInputSpecies}
\title{Return Miranda data for a given species.}
\usage{
getInputSpecies(
  selection,
  threshold = 60,
  energy = NULL,
  targetIden = NULL,
  mirnaIden = NULL
)
}
\arguments{
\item{selection}{Species (species selection are either for mature miRNA species "Human1","Mouse",
"C.elegans", "Epstein_Barr", "Epstein_Barr_Human",
"Drosophila","Kaposi_Sarcoma", "KSHV_Human",
"Cytomegalovirus","CMV_Human")}

\item{threshold}{miRanda score threshold default 60}

\item{energy}{miRanda folding energy threshold default NULL}

\item{targetIden}{miRanda target identity score default NULL}

\item{mirnaIden}{miRanda mirna identity score default NULL}
}
\value{
data.frame with Miranda data.
}
\description{
Reads Miranda file for a given speicies and returns it as
a data.frame, thresholded by percent identity.
Header options are Score (threshold), Energy-Kcal/Mol(energy),
Subject-IdentityPercent(targetIden), Query-IdentityPercent (mirnaIden)
}
\examples{
x <- getInputSpecies("Epstein_Barr", threshold = 60) # Default is threshold 60
}
\keyword{miranda,}
\keyword{species}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotResiduals.R
\name{plotResiduals}
\alias{plotResiduals}
\title{Plot residuals}
\usage{
plotResiduals(model)
}
\arguments{
\item{model}{linear model}
}
\value{
does not return value
}
\description{
Plot residuals description
}
\examples{
plotResiduals(lm(x ~ y, data = data.frame(x = runif(10), y = runif(10))))
}
\keyword{model}
\keyword{plot,}
\keyword{residuals,}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modelsFilter.R
\name{modelsFilter}
\alias{modelsFilter}
\title{modelsFilter Filter a list of models based on logical expression}
\usage{
modelsFilter(models, expr, quiet = FALSE)
}
\arguments{
\item{models}{list of models and related elemenets, such as returned by \code{runModelsZInf()}}

\item{expr}{expresion that yields a logical vector (evaluated in the environmnet of \code{model})}

\item{quiet}{suppress warnings}
}
\value{
\code{models} but with all elements filtered by logical expression \code{expr}. Elements
        for which filter could not be applied (e.g. length mismatch between element and condition)
        are set to \code{NA}.
}
\description{
This function can be used to filter a list of models (such as returned by \code{runModelsZInf()})
based on a logical expression.
}
\examples{
\donttest{
x <- modelsFilter(models, pvalues < 0.05)
x <- modelsFilter(models, is_significant)
x <- modelsFilter(models, is_significant == FALSE)
}
}
\keyword{R}
\keyword{filter}
\keyword{models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/models_multi.R
\name{glm_multi}
\alias{glm_multi}
\title{Model functions for GLM with negative binomial family.}
\usage{
glm_multi(
  models = c(glm_gaussian, glm_nb, glm_poisson, glm_zeroinfl_poisson,
    glm_zeroinfl_negbin)
)
}
\arguments{
\item{models}{Model type, one or more of glm_gaussian, glm_nb, glm_poisson, glm_zeroinfl_poisson or glm_zeroinfl_negbin}
}
\value{
structure containing functions \code{fit}, \code{coefficients},
        \code{aic}, \code{data}, \code{pterm}, \code{pmodel}, and a
        character string "glm_multi" in \code{model}.
}
\description{
Runs models 'glm_gaussian', 'glm_nb', 'glm_poisson',
'glm_zeroinfl(poisson)', 'glm_zeroinfl(negbin)' and returns
mode with lowest AIC.
}
\examples{
x <- glm_multi()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mirRnaDensityCor.R
\name{mirRnaDensityCor}
\alias{mirRnaDensityCor}
\title{mirRnaDensityCor for miRTarRNASeq miRNA and mRNA correlation real data versus sampled data}
\usage{
mirRnaDensityCor(corr0, corrS, pvalue = 0.05)
}
\arguments{
\item{corr0}{data.frame results of corMirnaRna function.}

\item{corrS}{data.frame results from the sampCorRnaMirna function.}

\item{pvalue}{The p value threshold to be used on the data density plot default is 0.05.}
}
\value{
Density plot
}
\description{
This function draws density plots for miRNA and mRNA correlation while
comparing real data vs sampled data. It mainly illustrates the where the lower %5 (sig)
relationships lie.
}
\examples{
x <- mirRnaDensityCor(corr_0, outs, pvalue = 0.05)
}
\keyword{Density}
\keyword{plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/models_poisson.R
\name{glm_poisson}
\alias{glm_poisson}
\title{Model functions for GLM with Poisson model.}
\usage{
glm_poisson()
}
\value{
structure containing functions \code{fit}, \code{coefficients},
        \code{aic}, \code{data}, \code{pterm}, \code{pmodel}, and a
        character string "glm_poisson" in \code{model}.
}
\description{
Implements standardaized functions to fit the glm with
Poisson family and to obtain coefficients, pvalues, etc.
}
\examples{
x <- glm_poisson()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeFormulaRightSide.R
\name{makeFormulaRightSide}
\alias{makeFormulaRightSide}
\title{makeFormulaRightSide makes right hand side of formula for model variables: vector of indep. variables}
\usage{
makeFormulaRightSide(variables, mode = "multi")
}
\arguments{
\item{variables}{The vector created by miRNA_select}

\item{mode}{One of "multi", "inter" or NULL}
}
\value{
data.frame containing Miranda data
}
\description{
This function make right hand side of formula for model variables: vector of indep. variables (i.e. miRNAs)
mode: 'multi' for simple, 'inter' for model with interactions returns a string in the form "~ a + b", or "~ a + b + a * b"
}
\examples{
\donttest{
x <- makeFormulaRightSide(variables, mode = "multi")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/models_run.R
\name{modelData}
\alias{modelData}
\title{Obtain model input data}
\usage{
modelData(x)
}
\arguments{
\item{x}{fitted model}
}
\value{
Input data for the fitted model
}
\description{
Obtain model input data
}
\examples{
x <- modelData(some_model)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/combiner.R
\name{combiner}
\alias{combiner}
\title{combiner combines the miRNA and mRNA files}
\usage{
combiner(mRNA, miRNA, miRNA_select)
}
\arguments{
\item{mRNA}{Matrix or data.frame mRNA/RNA from transforemed diff expression file (genrated
using TZtranz)}

\item{miRNA}{Matrix or data frame miRNA from transforemed diff file (genrated
using TZtranz)}

\item{miRNA_select}{A vector of character's for miRNAs which the user is interested in investigating if glm is use 1 miRNA
should be input. If multivariate several miRNAs should be imported, same goes for interaction determination for miRNAs. Note
we do not recommend more than 3-4 miRNAs at a time for the latter cases.}
}
\value{
A dataframe which includes only mRNAs and miRNA intersection for the next estimation geneVari output.
}
\description{
This function makes and intersection dataframe for mRNA and miRNA/s of interest to be tested.
}
\examples{
miRNA_select <- c("ebv-mir-bart9-5p")
x <- combiner(mRNA, miRNA, miRNA_select)
}
\keyword{miRNA_mRNA_Intersection}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/importMirandaFile.R
\name{downloadMirandaFile}
\alias{downloadMirandaFile}
\title{downloadMirandaFile Read internal Miranda file}
\usage{
downloadMirandaFile(urlf)
}
\arguments{
\item{urlf}{URL of the specific chosen file}
}
\value{
data.frame containing downloaded miRanda file
}
\description{
Reads internal Miranda file from extdata and returns it as a data.frame
}
\examples{
\donttest{
x <- downloadMirandaFile(
       "https://zenodo.org/record/4615670/files/Mouse_miRanda.txt.gz"
     )
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mirRnaDensityInter.R
\name{mirRnaDensityInter}
\alias{mirRnaDensityInter}
\title{mirRnaDensityInter for mirTarRnaSeq miRNA and mRNA Interrelation real data versus sampled data}
\usage{
mirRnaDensityInter(Inter0, OUTS, pvalue = 0.05)
}
\arguments{
\item{Inter0}{data.frame results of twoTimePoint function.}

\item{OUTS}{data.frame results from the twoTimePointSamp function.}

\item{pvalue}{The p value threshold to be used on the data density plot default is 0.05.}
}
\value{
Density plot
}
\description{
This function draws density plots for miRNA and mRNA Interrelation while
comparing real data vs sampled data. It mainly illustrates the where the lower %5 (sig)
relationships lie.
}
\examples{
\donttest{
x <- mirRnaDensityInter(Inter0, OUTS, pvalue = 0.05)
}

}
\keyword{Density}
\keyword{plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/models_run.R
\name{modelTermPvalues}
\alias{modelTermPvalues}
\title{Obtain p-values for terms in model formula}
\usage{
modelTermPvalues(x)
}
\arguments{
\item{x}{fitted model}
}
\value{
Pvalue for the terms in the fitted model
}
\description{
Obtain p-values for terms in model formula
}
\examples{
modelTermPvalues(some_model)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/corMirnaRna.R
\name{corMirnaRna}
\alias{corMirnaRna}
\title{corMirnaRna correlation for miRNA and mRNA}
\usage{
corMirnaRna(mRNA, miRNA, method = "pearson")
}
\arguments{
\item{mRNA}{mRNA file generated from foldchanges (FC) obj of the one2OneRnaMiRNA}

\item{miRNA}{miRNA file generated from foldchanges (FC) obj of the one2OneRnaMiRNA}

\item{method}{Default is "pearson" else use "kendall" or "spearman"}
}
\value{
Correlation data.frame
}
\description{
This function uses the output of one2OneRnaMiRNA and returns the correlation dataframe
}
\examples{
x <- corMirnaRna(mRNA_fc, miRNA_fc, method = "spearman")
}
\keyword{Correlation}
\keyword{Threshold}
\keyword{miRanda}
\keyword{miRanda,}
\keyword{with}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/corMirnaRnaMiranda.R
\name{corMirnaRnaMiranda}
\alias{corMirnaRnaMiranda}
\title{corMirnaRnaMiranda correlation for miRNA and mRNA}
\usage{
corMirnaRnaMiranda(mRNA, miRNA, CorVal, getInputSpeciesDF, method = "pearson")
}
\arguments{
\item{mRNA}{mRNA file generated from foldchanges (FC) obj of the one2OneRnaMiRNA.}

\item{miRNA}{miRNA file generated from foldchanges (FC) obj of the one2OneRnaMiRNA}

\item{CorVal}{Correlation cut off.Example: If correlation -0.2 it would only return correlations with
smaller than this value correlation for miRNA and mRNA at various time points.}

\item{getInputSpeciesDF}{The dataframe generated from the getInputSpecies function.}

\item{method}{Default is "pearson" else use "kendall" or "spearman".}
}
\value{
Correlation dataframe
}
\description{
This function uses the output of one2OneRnaMiRNA and returns the correlation dataframe.
}
\examples{
x <- corMirnaRnaMiranda(mRNA_fc, miRNA_fc, Cor = -0.9, miRandaM)
}
\keyword{Correlation}
\keyword{Threshold}
\keyword{miRanda}
\keyword{miRanda,}
\keyword{with}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{inter0}
\alias{inter0}
\title{This is data is the mRNA FC and miRNA FC correlation/interaction original data.
This data set is used in documentation examples.}
\description{
This is data is the mRNA FC and miRNA FC correlation/interaction original data.
This data set is used in documentation examples.
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/miRanComp.R
\name{miRanComp}
\alias{miRanComp}
\title{miRanComp comparison of mRNAs present with miRanda file targets}
\usage{
miRanComp(miRNA, miRanda)
}
\arguments{
\item{miRNA}{Matrix or data.frame miRNA/RNA file or transformed diff expression file (generated
using TZtranz)}

\item{miRanda}{A dataframe of miRanda file with miRNA$V1 and miRNA targets miRNA$V2}
}
\value{
An miRNA expression dataframe which includes only Genes/Targets present in miRanda file
}
\description{
This function generates a dataframe consisting of mRNA or miRNAs present in
 miRanda generated file using the miRTarRNASeq:::getInputSpecies() function
}
\examples{
x <- miRanComp(miRNA, miRanda)
}
\keyword{Intersection}
\keyword{expression}
\keyword{miRanda}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/models_run.R
\name{runModel}
\alias{runModel}
\title{Run a model of a specific kind}
\usage{
runModel(x, data, ..., model = glm_gaussian())
}
\arguments{
\item{x}{model formula}

\item{data}{data.frame to run the model on}

\item{...}{passed on to \code{fit()}}

\item{model}{model type}
}
\value{
fitted model
}
\description{
Run a model of a specific kind
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fdrSig.R
\name{fdrSig}
\alias{fdrSig}
\title{fdrSig Ruturns FDR significant miRNA/mRNA predictions}
\usage{
fdrSig(RMObj, value = 0.05, method = "fdr")
}
\arguments{
\item{RMObj}{The output of runModels}

\item{value}{The FDR value default is 0.1}

\item{method}{The p-value adjustment method default is fdr. It could be either of the following "holm", "hochberg",
"hommel","bonferroni", "BH", "BY", or "fdr".}
}
\value{
A list of FDR corrected p vlaues, annova, and significance for each gene and the miRNA/s of interest
}
\description{
This function performs FDR correction on the p_values generated by the runModels function list.
}
\examples{
models <- runModels(Combine, geneVariant, "ebv-mir-bart9-5p")
x <- fdrSig(models, value = 0.1, method = "fdr")
}
\keyword{correction}
\keyword{p_adjust,}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/importMirandaFile.R
\name{importMirandaFile}
\alias{importMirandaFile}
\title{importMirandaFile Read internal Miranda file}
\usage{
importMirandaFile(fn)
}
\arguments{
\item{fn}{filename}
}
\value{
data.frame containing Miranda data
}
\description{
Reads internal Miranda file from extdata and returns it as a data.frame
}
\examples{
\donttest{
x <- importMirandaFile("Mouse_miRanda.txt")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/miRandaIntersect.R
\name{miRandaIntersect}
\alias{miRandaIntersect}
\title{miRandaIntersect Looks for Intersection of Significant output results with miRanda Results from getInputSpeciesDF
function}
\usage{
miRandaIntersect(sig_corrs, corrS, mRNA, miRNA, getInputSpeciesDF)
}
\arguments{
\item{sig_corrs}{correlation matrix, produced by threshSig.}

\item{corrS}{vector of correlations/differences, from the sampCorRnaMirna function.}

\item{mRNA}{mRNA FC matrix.}

\item{miRNA}{miRNA FC matrix.}

\item{getInputSpeciesDF}{miranda data, produced by getInputSpecies.}
}
\value{
An object containing data.frames of significant mRNA, miRNA
            and correlation matrix filtered by miRanda input.
}
\description{
Compares and looks for intersection if significant output results with miRanda Results from getInputSpeciesDF and outputs a final
filtered output for only those pairs of miRNA and mRNA which have actually been predicted to be targets
in miRanda file
function
}
\examples{
x <- miRandaIntersect(sig_InterR, outs2, mRNA_fc, miRNA_fc, miRandaM)
}
\keyword{Significance,}
\keyword{Threshold,}
\keyword{intersect}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotTerms.R
\name{plotTerms}
\alias{plotTerms}
\title{plotTerms}
\usage{
plotTerms(model)
}
\arguments{
\item{model}{linear model}
}
\value{
does not return value
}
\description{
Plot terms description
}
\examples{
plotTerms(lm(x ~ y, data = data.frame(x = runif(10), y = runif(10))))
}
\keyword{model}
\keyword{plot,}
\keyword{residuals,}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/finInterResult.R
\name{finInterResult}
\alias{finInterResult}
\title{finInterResult miRNA and mRNA interrelation in two-time points results in a dataframe.}
\usage{
finInterResult(results)
}
\arguments{
\item{results}{Results from mirandaIntersectInter}
}
\value{
miRNA mRNA interelation dataframe
}
\description{
This function uses the output of one2OneRnaMiRNA and returns a sampled from orig file
interrelation dataframe depending on user sampling selection.
}
\examples{
x <- finInterResult(results)
}
\keyword{Results}
\keyword{dataframe}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{geneVariant}
\alias{geneVariant}
\title{This is data is the mRNA expression across samples and miRNA expression data which is to be
investigated giving directions on which data is miRNA and which is mRNA.
This data set is used in documentation examples.}
\description{
This is data is the mRNA expression across samples and miRNA expression data which is to be
investigated giving directions on which data is miRNA and which is mRNA.
This data set is used in documentation examples.
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sampCorRnaMirna.R
\name{sampCorRnaMirna}
\alias{sampCorRnaMirna}
\title{sampCorRnaMirna sampling for correlation for miRNA and mRNA}
\usage{
sampCorRnaMirna(
  mRNA,
  miRNA,
  method = "pearson",
  Shrounds = 100,
  Srounds = 1000
)
}
\arguments{
\item{mRNA}{mRNA file generated from fold changes (FC) obj of the one2OneRnaMiRNA.}

\item{miRNA}{miRNA file generated from fold changes (FC) obj of the one2OneRnaMiRNA.}

\item{method}{Default is "pearson" else use "kendall" or "spearman".}

\item{Shrounds}{number of shuffling over the FC data, default is 100.}

\item{Srounds}{number of sampling from the shuffled data, default is 1000.}
}
\value{
Correlation data frame
}
\description{
This function uses the output of one2OneRnaMiRNA and returns a sampled from original file
correlation dataframe depending on user sampling selection.
}
\examples{
x <- sampCorRnaMirna(mRNA_fc, miRNA_fc, method = "pearson", Shrounds = 10, Srounds = 10)
}
\keyword{correlation,}
\keyword{sampling,}
\keyword{shuffling}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mirRnaHeatmapDiff.R
\name{mirRnaHeatmapDiff}
\alias{mirRnaHeatmapDiff}
\title{mirRnaHeatmapDiff heatmap for miRTarRNASeq miRNA and mRNA correlation}
\usage{
mirRnaHeatmapDiff(
  finalF,
  ...,
  upper_bound = 0,
  main = "Default mRNA miRNA heatmap",
  color = c("grey90", viridis::inferno(50)),
  fontsize = 7
)
}
\arguments{
\item{finalF}{data.frame results of corMirnaRnaMiranda or corMirnaRna function}

\item{...}{arguments passed onto pheatmap}

\item{upper_bound}{is the upper_bound of the correlation pheatmap scale
default is zero user can set to values based on output of correlation result (value)}

\item{main}{is the title of the pheatmap}

\item{color}{default inferno(50) from the library viridis R base,
R colorbrewer and viridis compatible}

\item{fontsize}{default is 7 user adjustable}
}
\value{
pheatmap Obj
}
\description{
This function draws heatmaps (pheatmaps) for miRNA and mRNA correlation while
using default and heatmap for all other parameters
}
\examples{
x <- mirRnaHeatmapDiff(results$corrs, upper_bound = -0.1, color = rainbow(50), fontsize = 10)
}
\keyword{color,}
\keyword{correlation}
\keyword{heatmap,}
\keyword{pheatmap,}
\keyword{plot,correlation_plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/twoTimePoint.R
\name{twoTimePoint}
\alias{twoTimePoint}
\title{twoTimePoint miRNA and mRNA interrelation in two timepoints}
\usage{
twoTimePoint(mRNA, miRNA)
}
\arguments{
\item{mRNA}{mRNA file generated from fold changes (FC) obj of the one2OneRnaMiRNA.}

\item{miRNA}{miRNA file generated from fold changes (FC) obj of the one2OneRnaMiRNA.}
}
\value{
miRNA mRNA interrelation dataframe
}
\description{
This function uses the output of one2OneRnaMiRNA and returns a sampled from original file
interrelation dataframe depending on user sampling selection.
}
\examples{
\donttest{
x <- twoTimePoint(mRNA_fc2, miRNA_fc2)
}
}
\keyword{interrelation}
\keyword{mRNA}
\keyword{miRNA}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/one2manySponge.R
\name{one2manySponge}
\alias{one2manySponge}
\title{Sparse Partial Correlations On mRNA/miRNA Expression
We make mirTarRnaSeq compatible to SPONGE package in order to estimate
sparse matrix correlation (using elstic net) for prediction potential miRNA-mRNA
interaction. Note this function/method is suggested for miRNA/mRNA interactions
in many samples with a notable variance of mRNA/miRNA expression. This model also
only reports negative sparse partial correlation predictions.}
\usage{
one2manySponge(mirna_exp, diff_exp, miranda_sponge_predict, non_null = TRUE)
}
\arguments{
\item{mirna_exp}{miRna expression data.frame with miRNA for rows and samples for columns}

\item{diff_exp}{mRNA expression data.frame with mRNA for rows and samples for columns}

\item{miranda_sponge_predict}{miRanda sponge compatible matrix produced by miranda_sponge_predict function}

\item{non_null}{The default for this parameter is TRUE, hence it returns only non-null estimated if
FALSE it would return all NULL and TRUE estimates.}
}
\value{
matrix adjacency matrix with column names miRNA and row names mRNA
}
\description{
Sparse Partial Correlations On mRNA/miRNA Expression
We make mirTarRnaSeq compatible to SPONGE package in order to estimate
sparse matrix correlation (using elstic net) for prediction potential miRNA-mRNA
interaction. Note this function/method is suggested for miRNA/mRNA interactions
in many samples with a notable variance of mRNA/miRNA expression. This model also
only reports negative sparse partial correlation predictions.
}
\keyword{Sponge,}
\keyword{ceRNA}
\keyword{correlation,}
\keyword{mirTarRnaSeq,}
\keyword{partial}
\keyword{sparse}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geneVari.R
\name{geneVari}
\alias{geneVari}
\title{geneVari Makes a list of gene names to be used in the runModels function}
\usage{
geneVari(Combined, miRNA_select)
}
\arguments{
\item{Combined}{the combined file for mRNA and selected miRNAs output of combiner function}

\item{miRNA_select}{The vector of selected miRNA/s}
}
\value{
A vector of characters with defined mRNA dimensions
}
\description{
This function defines the boudnaries of mRNA vs miRNAs of interest to be analysed by the runModels function
}
\examples{
x <- geneVari(Combine, "ebv-mir-bart9-5p")
}
\keyword{dimensions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotFit.R
\name{plotFit}
\alias{plotFit}
\title{Plot model}
\usage{
plotFit(model)
}
\arguments{
\item{model}{linear model}
}
\value{
does not return value
}
\description{
Plot 2D description
}
\examples{
plotFit(lm(x ~ y, data = data.frame(x = runif(10), y = runif(10))))
}
\keyword{2d,}
\keyword{model}
\keyword{plot,}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/models_run.R
\name{canonicalModel_}
\alias{canonicalModel_}
\title{Decifer a 'model parameter' and run appropriate \code{glm_...} function.}
\usage{
canonicalModel_(model)
}
\arguments{
\item{model}{string, function or object representing a model type.}
}
\value{
model type object
}
\description{
Return canonical model from model type string, function of object. Returns
a model as returned by \code{glm_gaussian()} and others, based on a string,
function or model type object (i.e. \code{"glm_gaussian"}, \code{glm_gaussian} or
\code{glm_gaussian()}).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/twoTimePointSamp.R
\name{twoTimePointSamp}
\alias{twoTimePointSamp}
\title{twoTimePointSamp miRNA and mRNA interrelation in two timepoints sampling}
\usage{
twoTimePointSamp(mRNA, miRNA, Shrounds = 100, Srounds = 1000)
}
\arguments{
\item{mRNA}{mRNA file generated from fold changes (FC) obj of the one2OneRnaMiRNA.}

\item{miRNA}{miRNA file generated from fold changes (FC) obj of the one2OneRnaMiRNA.}

\item{Shrounds}{number of shuffling over the FC data, default is 100.}

\item{Srounds}{number of sampling from the shuffled data, default is 1000.}
}
\value{
miRNA mRNA interrelation dataframe
}
\description{
This function uses the output of one2OneRnaMiRNA and returns a sampled from orig file
interrelation dataframe depending on user sampling selection.
}
\examples{
\donttest{
x <- twoTimePointSamp(mRNA, miRNA, Shrounds = 10, Srounds = 10)
}
}
\keyword{correlation,}
\keyword{sampling,}
\keyword{shuffling}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/threshSig.R
\name{threshSig}
\alias{threshSig}
\title{threshSig Using shuffling threshold finds appropriate significant miRNA-mRNA correlation}
\usage{
threshSig(corr0, corrS, pvalue = 0.05)
}
\arguments{
\item{corr0}{data.frame results of corMirnaRna function.}

\item{corrS}{vector of correlations, from the sampCorRnaMirna function.}

\item{pvalue}{The p value threshold to be used on the sampled data.}
}
\value{
A dataframe of Significant mRNA and miRNA
}
\description{
This function uses the sampCorRnaMirna shuffled output to determine an appropriate threshold
for significant mRNA and miRNA relationship of the dataset and shows all those with significant
relationships.
}
\examples{
\donttest{
x <- mirRnaHeatmap(outs, corr_0)
}

}
\keyword{Significance,}
\keyword{Threshold}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/models_run.R
\name{modelModelName}
\alias{modelModelName}
\title{Obtain model name}
\usage{
modelModelName(x)
}
\arguments{
\item{x}{fitted model}
}
\value{
model name
}
\description{
Obtain model name
}
\examples{
modelModelName(some_model)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{mRNA0_2}
\alias{mRNA0_2}
\title{This is data is the mRNA0_2 FC for 0-2 time point.
This data set is used in documentation examples.}
\description{
This is data is the mRNA0_2 FC for 0-2 time point.
This data set is used in documentation examples.
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/drawCorPlot.R
\name{drawCorPlot}
\alias{drawCorPlot}
\title{drawCorPlot correlation plots for mRNA and miRNA regression results}
\usage{
drawCorPlot(corMatrix, ...)
}
\arguments{
\item{corMatrix}{Significant correlation matrix}

\item{...}{parameters form the corrplot package}
}
\value{
miRNA mRNA target correlation plot
}
\description{
This function plots correlations for mRNA and miRNAs regression results (negative correlation for multi and
 individual interactions and positive and negative for interactions)
}
\examples{
\donttest{
x <- drawCorPlot(corMatrix)
}

}
\keyword{R}
\keyword{correlation}
\keyword{plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/models_zeroinfl.R
\name{glm_zeroinfl_poisson}
\alias{glm_zeroinfl_poisson}
\title{alias for glm_zeroinfl("poisson")}
\usage{
glm_zeroinfl_poisson(...)
}
\arguments{
\item{...}{passed to \code{glm_zeroinfl}}
}
\value{
structure containing functions \code{fit}, \code{coefficients},
        \code{aic}, \code{data}, \code{pterm}, \code{pmodel}, and a
        character string "glm_zeroinfl" in \code{model}.
}
\description{
alias for glm_zeroinfl("poisson")
}
\examples{
x <- glm_zeroinfl_poisson()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/models_run.R
\name{modelCoefficients}
\alias{modelCoefficients}
\title{Obtain coefficients}
\usage{
modelCoefficients(x)
}
\arguments{
\item{x}{fitted model}
}
\value{
fitted model coefficients
}
\description{
Obtain coefficients
}
\examples{
modelCoefficients(some_model)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/models_run.R
\name{modelAIC}
\alias{modelAIC}
\title{Obtain model AIC}
\usage{
modelAIC(x)
}
\arguments{
\item{x}{fitted model}
}
\value{
AIC for model
}
\description{
Obtain model AIC
}
\examples{
modelAIC(some_model)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mirandaIntersectInter.R
\name{mirandaIntersectInter}
\alias{mirandaIntersectInter}
\title{mirandaIntersectInter Looks for Intersection of Significant output results with miRanda Results from getInputSpeciesDF
function}
\usage{
mirandaIntersectInter(sig_corrs, corrS, mRNA, miRNA, getInputSpeciesDF)
}
\arguments{
\item{sig_corrs}{correlation matrix, produced by threshSig}

\item{corrS}{vector of Differences/Correlations, from the sampCorRnaMirna function.}

\item{mRNA}{mRNA FC matrix.}

\item{miRNA}{miRNA FC matrix.}

\item{getInputSpeciesDF}{miranda data, produced by getInputSpecies.}
}
\value{
An object containing data.frames of significant mRNA, miRNA
            and correlation matrix filtered by miranda input.
}
\description{
Compares and looks for intersection if significant output results with miRanda Results from getInputSpeciesDF and outputs a final
filterd ourput for only those pairs of miRNA and mRNA which have actually been predicted to be targets
in miRanda file
function
}
\examples{
x <- mirandaIntersectInter(sig_InterR, outs2, mRNA_fc2, miRNA_fc2, miRandaM)
}
\keyword{Threshold}
\keyword{intersect}
\keyword{miRanda}
\keyword{mirandaIntersectInter}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tzTrans.R
\name{tzTrans}
\alias{tzTrans}
\title{tzTransTranspose and z-score transformation}
\usage{
tzTrans(x)
}
\arguments{
\item{x}{matrix of miRNA or mRNA or the data frame to be transformed}
}
\value{
transposed and transformed version of x as a matrix.
}
\description{
Transposes and z-score transforms a matrix or data.frame.
}
\examples{
x <- tzTrans(miRNA)
}
\keyword{scale,}
\keyword{transform,}
\keyword{transpose}
\keyword{zscore,}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/one2OneRnaMiRNA.R
\name{one2OneRnaMiRNA}
\alias{one2OneRnaMiRNA}
\title{one2OneRnaMiRNA correlation for miRNA and mRNA  using differntial expression
fold change and if/when available p-value}
\usage{
one2OneRnaMiRNA(
  files,
  gene_colname = "Gene",
  fc_colname = "FC",
  pval_colname = "pvalue",
  pthreshold = NULL
)
}
\arguments{
\item{files}{a list of dataframes either miRNAs or mRNAs from various time points.}

\item{gene_colname}{Default is a vector character of length 1 "Gene" user can alter if they choose
This column contains the gene names.}

\item{fc_colname}{Default "FC" is coloumn name for fold changes user can alter if they choose.}

\item{pval_colname}{Default is "pvalue" column name for p-values (in input).}

\item{pthreshold}{P-value threshold.}
}
\value{
Correlation dataframe
}
\description{
This function inputs accept a list of dataframes and returns an obj with
two dataframes called FC and p-value. FC with rownames == genes and columns are FC1, 2, 3, ...
(with fold-changes) - P-value with rownames == genes and columns are P1, 2, 3, ... (with p-values)
both data.frames have the same order dimensions.
}
\examples{
\donttest{
x <- one2OneRnaMiRNA(files)
}
}
\keyword{correlation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sponge.R
\name{miranda_sponge_predict}
\alias{miranda_sponge_predict}
\title{Transform miRanda data for relevant mRNA and miRNA to matrix form compatible with sponge}
\usage{
miranda_sponge_predict(mirna_exp, diff_exp, miranda_data)
}
\arguments{
\item{mirna_exp}{miRna expression data.frame with miRNA for rows and samples for columns}

\item{diff_exp}{mRNA expression data.frame with mRNA for rows and samples for columns}

\item{miranda_data}{miRanda data.frame with the first two columns having miRNA and mRNA names}
}
\value{
matrix adjacency matrix with column names miRNA and row names mRNA
}
\description{
Transforms miRanda data into adjacency matrix, with 1 indicating
presence of a relationship between a mRNA and miRNA, and 0
otherwise. miRanda input is filtered by miRNA and mRNA present in
`mirna_exp` and `diff_expr` row names, respectively.
}
\keyword{SPONGE}
\keyword{ceRNA,}
\keyword{correlation,}
\keyword{miRanda,}
\keyword{partial}
\keyword{sparse}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runAllMirnaModels.R
\name{runAllMirnaModels}
\alias{runAllMirnaModels}
\title{runAllMirnaModels runModel for all miRNAs}
\usage{
runAllMirnaModels(
  mirnas,
  DiffExpmRNA,
  DiffExpmiRNA,
  miranda_data,
  prob = 0.75,
  fdr_cutoff = 0.1,
  method = "fdr",
  cutoff = 0.05,
  all_coeff = FALSE,
  mode = NULL,
  family = glm_poisson(),
  scale = 1
)
}
\arguments{
\item{mirnas}{vector of unique miRNAs under investigation.}

\item{DiffExpmRNA}{deferentially/expressed mRNAs expression file.}

\item{DiffExpmiRNA}{deferentially/expressed miRNAs expression file.}

\item{miranda_data}{getInputSpecies output file ( use low filters).}

\item{prob}{user defined ratio for miRanda distribution for miRanda score selection default is 0.75.}

\item{fdr_cutoff}{cutoff for FDR selection default is 0.1.}

\item{method}{finInterResult miRNA and mRNA interrelation in two time points  results in a dataframe.}

\item{cutoff}{P-value cutoff of the model.}

\item{all_coeff}{if true only models with all negative coefficients will be selected if false at least one
negative coefficient should be in the model; default is TRUE.}

\item{mode}{model mode, default is Null, can be changed to "multi" and "inter".}

\item{family}{Default is glm_poisson(), for zero inflated negative binomial NB option use glm_zeroinfl(dist="negbin").}

\item{scale}{if normalized data (FPKM,RPKM,TPM,CPM), scale to 10 etc., however the higher you go on
#scale the less accuracy your p-value estimate will be.}
}
\value{
List of run models
}
\description{
This function runs the "runModel" function for all miRNAs and mRNA combinations of two and returns a
list with significant genes and FDR models
}
\examples{
mirnas <- c("ebv-mir-bart9-5p", "ebv-mir-bart6-3p")
x <- runAllMirnaModels(mirnas, mRNA, miRNA, miRanda,
    prob = 0.90, fdr_cutoff = 0.1, method = "fdr",
    all_coeff = TRUE, mode = "multi",
    family = glm_poisson(), scale = 100
)
}
\keyword{all_miRNAs}
\keyword{runAllMirnaModels}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runModels.R
\name{runModels}
\alias{runModels}
\title{runModels runs miRNA mrna model model for various miRNA-mRNA data distributions}
\usage{
runModels(
  combination,
  select_mRNA,
  select_miRNA,
  mode = NULL,
  family = glm_poisson(),
  scale = 1,
  cutoff = 0.05,
  all_coeff = NULL
)
}
\arguments{
\item{combination}{the combined file for mRNA and selected miRNAs output of combiner function}

\item{select_mRNA}{the output of gene_variant function.}

\item{select_miRNA}{The vector of miRNA/s to be investigated.}

\item{mode}{the mode of analysis if more than one miRNA is being investigated multivariate "multi"
or co-variate/interaction analysis "inter" is being used}

\item{family}{gaussian or poisson}

\item{scale}{factor to scale input data (for genes) by, prior to rounding and model
fitting. (\code{scale} must be greater than zero).}

\item{cutoff}{p-value cut off to call significance}

\item{all_coeff}{if true only models with all negative coefficients will be selected if false at least one}
}
\value{
A list of p-vlaues, annova, and significance for each gene and the miRNA/s of interest
}
\description{
This function defines the boundaries of mRNA vs miRNAs of interest to be analysed by the runModels function
}
\examples{
x <- runModels(Combine, geneVariant, "ebv-mir-bart9-5p")
}
\keyword{glm}
\keyword{interaction}
\keyword{multivariate}
\keyword{runModels}
\keyword{univariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/drawInterPlots.R
\name{drawInterPlots}
\alias{drawInterPlots}
\title{drawInterPlots for finInterResult miRNA and mRNA Interrelation real data}
\usage{
drawInterPlots(mrna, mirna, final_results)
}
\arguments{
\item{mrna}{mRNA results of twoTimePoint function.}

\item{mirna}{miRNA results of twoTimePoint function.}

\item{final_results}{finInterResult miRNA and mRNA interrelation in two timepoints  results in a dataframe.}
}
\value{
par plots
}
\description{
This function draws miRNA, mRNA density plots for miRNA and mRNA Interrelation while
comparing in addition to overall FC_miRNA and FC_mRNA plots from the finInterResult dataframe function.
}
\examples{
x <- drawInterPlots(mRNA_fc2, miRNA_fc2, final_results)
}
\keyword{plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{mRNA_fc2}
\alias{mRNA_fc2}
\title{This data is the mRNA fold change data set for difference or interrelation section.
This data set is used in documentation examples.}
\description{
This data is the mRNA fold change data set for difference or interrelation section.
This data set is used in documentation examples.
}
\keyword{data}
